@@ -11,11 +11,11 @@
  */
 void
 error(str)
-char	*str;	/* must be a constant string, temporary strings don't work */
+const char	*str;
 {
-	error_str = str;
+	error_str = str;	/* must be a constant string, temporary strings don't work */
 #if	!SILENT
-	set_color(2);	/* set color to red */
+	set_color(2);		/* set color to red */
 	printf("%s\n", str);
 	default_color();
 #endif
@@ -50,6 +50,8 @@ error_huge()
 
 /*
  * Check if a floating point math function flagged an error.
+ *
+ * There is no return if an error message is displayed.
  */
 void
 check_err()
@@ -73,7 +75,9 @@ check_err()
 }
 
 /*
- * Allocate the needed temp storage.
+ * Allocate the needed global expression storage arrays.
+ * Each has "n_tokens" elements.
+ * "n_tokens" must not change until Mathomatic terminates.
  *
  * Return true if successful.
  */
@@ -102,13 +106,15 @@ init_gvars()
 	high_prec = false;
 	partial_flag = true;
 	symb_flag = false;
-	sign_flag = false;
+	sign_cmp_flag = false;
 	approximate_roots = false;
 
+	/* initialize the universal and often used constant "0" expression */
 	zero_token.level = 1;
 	zero_token.kind = CONSTANT;
 	zero_token.token.constant = 0.0;
 
+	/* initialize the universal and often used constant "1" expression */
 	one_token.level = 1;
 	one_token.kind = CONSTANT;
 	one_token.token.constant = 1.0;
@@ -130,19 +136,6 @@ clean_up()
 	}
 }
 
-/*
- * Restart sign array.
- */
-void
-clear_sign_array()
-{
-	int	i;
-
-	for (i = 0; i < ARR_CNT(sign_array); i++) {
-		sign_array[i] = false;
-	}
-}
-
 /*
  * Register all sign variables in all equation spaces.
  */
@@ -151,18 +144,16 @@ set_sign_array()
 {
 	int	i, j;
 
-	clear_sign_array();
+	CLEAR_ARRAY(sign_array);
 	for (i = 0; i < n_equations; i++) {
 		if (n_lhs[i] > 0) {
 			for (j = 0; j < n_lhs[i]; j += 2) {
-				if (lhs[i][j].kind == VARIABLE
-				    && (lhs[i][j].token.variable & VAR_MASK) == SIGN) {
+				if (lhs[i][j].kind == VARIABLE && (lhs[i][j].token.variable & VAR_MASK) == SIGN) {
 					sign_array[(lhs[i][j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 				}
 			}
 			for (j = 0; j < n_rhs[i]; j += 2) {
-				if (rhs[i][j].kind == VARIABLE
-				    && (rhs[i][j].token.variable & VAR_MASK) == SIGN) {
+				if (rhs[i][j].kind == VARIABLE && (rhs[i][j].token.variable & VAR_MASK) == SIGN) {
 					sign_array[(rhs[i][j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 				}
 			}
@@ -180,7 +171,7 @@ long	*vp;
 {
 	int	i;
 
-	for (i = 2;; i++) {
+	for (i = 0;; i++) {
 		if (i >= ARR_CNT(sign_array)) {
 			/* out of unique sign variables */
 			*vp = SIGN;
@@ -204,23 +195,44 @@ clear_all()
 {
 	int	i;
 
-/* set to equation space number 1 */
+/* select first equation space */
 	cur_equation = 0;
-/* erase all equation spaces */
-	for (i = 0; i < n_equations; i++) {
-		n_lhs[i] = 0;
-		n_rhs[i] = 0;
-	}
+/* erase all equation spaces by setting their length to zero */
+	CLEAR_ARRAY(n_lhs);
+	CLEAR_ARRAY(n_rhs);
 /* forget all variables names */
 	for (i = 0; var_names[i]; i++) {
 		free(var_names[i]);
 		var_names[i] = NULL;
 	}
-/* reset everything to be in a known state */
-	clear_sign_array();
+/* reset everything to a known state */
+	CLEAR_ARRAY(sign_array);
 	init_gvars();
 }
 
+/*
+ * Return true if the specified equation space is available, allocating if necessary.
+ */
+int
+alloc_espace(i)
+int	i;	/* equation space number */
+{
+	if (i >= N_EQUATIONS)
+		return false;
+	if (lhs[i] && rhs[i])
+		return true;	/* already allocated */
+	lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
+	if (lhs[i] == NULL)
+		return false;
+	rhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
+	if (rhs[i] == NULL) {
+		free(lhs[i]);
+		lhs[i] = NULL;
+		return false;
+	}
+	return true;
+}
+
 /*
  * Allocate or reuse an empty equation space.
  *
@@ -234,17 +246,7 @@ alloc_next_espace()
 	for (n = cur_equation, i = 1;; n = (n + 1) % n_equations, i++) {
 		if (i > n_equations) {
 			i = n_equations;
-			if (i < N_EQUATIONS) {
-				lhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
-				if (lhs[i]) {
-					rhs[i] = (token_type *) malloc(n_tokens * sizeof(token_type));
-					if (rhs[i] == NULL) {
-						free(lhs[i]);
-						lhs[i] = NULL;
-					}
-				}
-			}
-			if (i >= N_EQUATIONS || lhs[i] == NULL) {
+			if (!alloc_espace(i)) {
 				return -1;
 			}
 			n_equations++;
@@ -258,7 +260,7 @@ alloc_next_espace()
 }
 
 /*
- * Return the number of the next empty equation space.
+ * Return the number of the next empty equation space, otherwise don't return.
  */
 int
 next_espace()
@@ -317,7 +319,7 @@ long		v;	/* standard Mathomatic variable */
 	int	j;
 
 	if (v) {
-		for (j = 0; j < n; j += 2) {
+		for (j = 0; j < n; j++) {
 			if (p1[j].kind == VARIABLE && p1[j].token.variable == v) {
 				return true;
 			}
@@ -357,6 +359,8 @@ long		v;		/* variable to substitute with expression */
 	int	j, k;
 	int	level;
 
+	if (v == 0 || len <= 0)
+		return;
 	for (j = *np - 1; j >= 0; j--) {
 		if (equation[j].kind == VARIABLE && equation[j].token.variable == v) {
 			level = equation[j].level;
@@ -428,6 +432,7 @@ char	*cp;
  * Parse an expression with equation space pull if the line starts with "#".
  *
  * Returns the new string position or NULL on error.
+ * Currently, there can be no more to parse in the string when this returns.
  */
 char	*
 parse_expr(equation, np, cp)
@@ -438,6 +443,9 @@ char		*cp;		/* string to parse */
 	int	i;
 	char	*cp1, *cp2;
 
+	if (!case_sensitive_flag) {
+		str_tolower(cp);
+	}
 	cp1 = skip_space(cp);
 	if (*cp1 == '#') {
 		cp1++;
@@ -470,7 +478,11 @@ char		*cp;		/* string to parse */
 		fprintf(gfp, "\n");
 		return cp2;
 	}
-	return parse_section(equation, np, cp);
+	if ((cp = parse_section(equation, np, cp)) != NULL) {
+		if (extra_characters(cp))
+			return NULL;
+	}
+	return cp;
 }
 
 /*
@@ -491,11 +503,8 @@ int		*np;		/* pointer to the returned parsed expression length */
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
 		cp = parse_expr(equation, np, cp);
-		if (cp && !extra_characters(cp)) {
+		if (cp) {
 			break;
 		}
 	}
@@ -593,15 +602,15 @@ int	n;		/* maximum size of "string" in bytes */
 	input_column = strlen(prompt_str);
 #if	READLINE
 	if (readline_enabled) {
-		rl_inhibit_completion = true;	/* turn off readline file name completion */
 		cp = readline(prompt_str);
 		if (cp == NULL) {
 			fprintf(gfp, _("\nEnd of input.\n"));
 			exit_program(0);
 		}
 		my_strlcpy(string, cp, n);
-		if (string[0]) {
+		if (skip_space(cp)[0] && (last_history_string == NULL || strcmp(last_history_string, cp))) {
 			add_history(cp);
+			last_history_string = cp;
 		} else {
 			free(cp);
 		}
@@ -619,11 +628,14 @@ int	n;		/* maximum size of "string" in bytes */
 		exit_program(0);
 	}
 #endif
+	// Fix an fgets() peculiarity:
 	i = strlen(string) - 1;
 	if (i >= 0 && string[i] == '\n') {
 		string[i] = '\0';
 	}
 	if (gfp != stdout) {
+		/* Input that is prompted for is now included in the redirected output of a command to a file,
+		   making redirection like logging. */
 		fprintf(gfp, "%s%s\n", prompt_str, string);
 	}
 	set_error_level(string);
@@ -697,7 +709,7 @@ char	*cp;
 
 /*
  * Process an equation number range given in text string "*cpp".
- * Skip past all spaces and update "*cpp" to point to the next argument.
+ * Skip past all spaces and update "*cpp" to point to the next argument if successful.
  * If no equation number or range is given, assume the current equation is wanted.
  *
  * Return true if successful,
@@ -710,28 +722,18 @@ char	**cpp;
 int	*ip, *jp;
 {
 	int	i;
+	char	*cp;
 
-	*cpp = skip_space(*cpp);
-#if	LIBRARY		/* only allow a single equation number when using as a library */
-	if (isdigit(**cpp)) {
-		i = decstrtol(*cpp, cpp) - 1;
-	} else {
-		i = cur_equation;
-	}
-	if (not_defined(i)) {
-		return false;
-	}
-	*ip = i;
-	*jp = i;
-	return true;
-#else
-	if (is_all(*cpp)) {
-		*cpp = skip_param(*cpp);
+	cp = skip_space(*cpp);
+	if (is_all(cp)) {
+		cp = skip_param(cp);
 		*ip = 0;
 		*jp = n_equations - 1;
+		while (*jp > 0 && n_lhs[*jp] == 0)
+			(*jp)--;
 	} else {
-		if (isdigit(**cpp)) {
-			*ip = strtol(*cpp, cpp, 10) - 1;
+		if (isdigit(*cp)) {
+			*ip = strtol(cp, &cp, 10) - 1;
 		} else {
 			*ip = cur_equation;
 		}
@@ -739,17 +741,17 @@ int	*ip, *jp;
 			error(_("Invalid equation number."));
 			return false;
 		}
-		if (**cpp != '-') {
+		if (*cp != '-') {
 			if (not_defined(*ip)) {
 				return false;
 			}
 			*jp = *ip;
-			*cpp = skip_space(*cpp);
+			*cpp = skip_space(cp);
 			return true;
 		}
-		(*cpp)++;
-		if (isdigit(**cpp)) {
-			*jp = strtol(*cpp, cpp, 10) - 1;
+		(cp)++;
+		if (isdigit(*cp)) {
+			*jp = strtol(cp, &cp, 10) - 1;
 		} else {
 			*jp = cur_equation;
 		}
@@ -763,28 +765,27 @@ int	*ip, *jp;
 			*jp = i;
 		}
 	}
-	*cpp = skip_space(*cpp);
+	cp = skip_space(cp);
 	for (i = *ip; i <= *jp; i++) {
 		if (n_lhs[i] > 0) {
+			*cpp = cp;
 			return true;
 		}
 	}
 	error(_("No equations defined in specified range."));
 	return false;
-#endif
 }
 
 /*
- * This function is provided to make sure there is nothing else
- * on a command line.
+ * This function is provided to make sure there is nothing else on a command line.
  *
- * Returns false if OK.
- * Returns true if any non-space characters were encountered
- * and an error message was printed.
+ * Returns true if any non-space characters are encountered before the end of the string
+ * and an error message is printed.
+ * Otherwise just returns false.
  */
 int
 extra_characters(cp)
-char	*cp;	/* command line */
+char	*cp;	/* command line string */
 {
 	if (cp) {
 		cp = skip_space(cp);
@@ -797,7 +798,8 @@ char	*cp;	/* command line */
 }
 
 /*
- * get_range() if it is the last possible option on the command line.
+ * get_range() if it is the last possible option on the command line,
+ * otherwise display an error message and return false.
  */
 int
 get_range_eol(cpp, ip, jp)
@@ -865,7 +867,7 @@ char	*cp;	/* character pointer */
 }
 
 /*
- * Compare strings up to the first space.
+ * Compare strings up to the end or the first space.
  * Must be an exact match.
  * Compare is alphabetic case insensitive.
  *
@@ -1029,3 +1031,17 @@ int		n1;	/* expression length */
 	}
 	return true;
 }
+
+/*
+ * Check for division by zero.
+ */
+void
+check_divide_by_zero(denominator)
+double	denominator;
+{
+#if	!TRAP_FPERRORS
+	if (denominator == 0) {
+		warning(_("Division by zero."));
+	}
+#endif
+}
