@@ -1,16 +1,9 @@
 /*
- * Main include file for Mathomatic.
+ * Main include file for Mathomatic, an algebraic manipulator written entirely in C.
  *
  * Copyright (C) 1987-2008 George Gesslein II.
  */
 
-#define	true	1
-#define	false	0
-
-#if	LIBRARY
-#define	SILENT	1
-#endif
-
 #ifndef	max
 #define max(a, b)    (((a) > (b)) ? (a) : (b))		/* return the maximum of two values */
 #endif
@@ -26,6 +19,7 @@
 #endif
 
 #define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
+#define CLEAR_ARRAY(a)	memset(a, 0, sizeof(a))			/* sets all elements of an array to zero */
 
 #define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
 #define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
@@ -33,7 +27,7 @@
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N	/* internationalization */
+#if	I18N	/* internationalization: allow other languages if all strings are translated; not tested */
 #define _(str)		gettext(str)
 #else
 #define _(str)		str
@@ -42,7 +36,7 @@
 #define	STANDARD_SCREEN_COLUMNS	80			/* default number of columns of characters on the screen */
 #define	STANDARD_SCREEN_ROWS	24			/* default number of lines on the screen */
 
-#define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temp file template */
+#define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temporary file template for mkstemp(3) */
 
 #define	PROMPT		"-> "				/* user interface prompt string */
 #define	HTML_PROMPT	"&mdash;&gt; "
@@ -53,7 +47,11 @@
 /*
  * The following defines the maximum number of equation spaces that can be allocated.
  */
+#if	HANDHELD
+#define	N_EQUATIONS	20
+#else
 #define	N_EQUATIONS	100
+#endif
 
 /*
  * The following defines the default maximum mathematical expression size.
@@ -63,10 +61,18 @@
  * This should be made much smaller for handhelds and embedded systems.
  * Do not set to less than 100.
  */
+#if	HANDHELD
+#define	DEFAULT_N_TOKENS	300
+#else
 #define	DEFAULT_N_TOKENS	30000
+#endif
 
-#define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
+#define	DIVISOR_SIZE		(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
+/*
+ * All Mathomatic variables are referenced by the long data type.
+ * The actual name string is stored separately.
+ */
 #define	MAX_VAR_NAMES	8000			/* maximum number of variable names, keep this under (VAR_MASK - VAR_OFFSET) */
 #define	MAX_VAR_LEN	100			/* maximum number of characters in variable names */
 
@@ -76,9 +82,9 @@
 #define	VAR_MASK	0x3fffL			/* mask for bits containing a reference to the variable name */
 #define	VAR_SHIFT	14			/* number of bits set in VAR_MASK */
 #define	SUBSCRIPT_MASK	63			/* mask for variable subscript after shifting VAR_SHIFT */
-#define	MAX_SUBSCRIPT	(SUBSCRIPT_MASK - 1)	/* maximum variable subscript */
+#define	MAX_SUBSCRIPT	(SUBSCRIPT_MASK - 1)	/* maximum variable subscript, currently only used for "sign" variables */
 
-typedef	char	sign_array_type[MAX_SUBSCRIPT+2];
+typedef	char	sign_array_type[MAX_SUBSCRIPT+2];	/* boolean array for generating unique "sign" variables */
 
 enum kind_list {		/* kinds of tokens specified in the union below */
 	CONSTANT,
@@ -89,7 +95,7 @@ enum kind_list {		/* kinds of tokens specified in the union below */
 typedef union {
 	double	constant;	/* internal storage for mathematical constants */
 	long	variable;	/* internal storage for mathematical variables */
-/* predefined special variables follow (order is important) */
+/* Predefined special variables follow (order is important): */
 #define	V_NULL		0L	/* null variable, should never be stored in an expression */
 #define	V_E		1L	/* the constant "e" or "e#" */
 #define	V_PI		2L	/* the constant "pi" or "pi#" */
@@ -97,7 +103,7 @@ typedef union {
 #define	SIGN		4L	/* for "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
 	int	operatr;	/* internal storage for mathematical operators */
-/* valid operators follow (order doesn't matter) */
+/* Valid operators follow (order doesn't matter), 0 is reserved for no operator: */
 #define	PLUS		1	/* a + b */
 #define	MINUS		2	/* a - b */
 #define	TIMES		3	/* a * b */
@@ -116,7 +122,7 @@ typedef	struct {		/* storage structure for each token in an expression */
 } token_type;
 
 typedef struct {		/* variable sort data structure */
-	long	v;		/* variable */
+	long	v;		/* Mathomatic variable */
 	int	count;		/* number of times the variable occurs */
 } sort_type;
 
