@@ -36,11 +36,14 @@ int
 version_cmd(cp)
 char	*cp;	/* the command line argument */
 {
+	int		rv;	/* return value */
+
 #if	LIBRARY
 	free(result_str);
 	result_str = strdup(VERSION);
 #endif
-	return version_report();
+	rv = version_report();
+	return rv;
 }
 
 /*
@@ -49,7 +52,7 @@ char	*cp;	/* the command line argument */
 int
 version_report()
 {
-	fprintf(gfp, _("Mathomatic version %s, written by George Gesslein II.\nCompile flags used: "), VERSION);
+	fprintf(gfp, _("Mathomatic version %s\nCompile flags used: "), VERSION);
 #if	__STDC__
 	fprintf(gfp, "__STDC__ ");
 #endif
@@ -77,7 +80,7 @@ version_report()
 #if	I18N
 	fprintf(gfp, "I18N ");
 #endif
-	fprintf(gfp, _("\nMaximum memory usage: %lld kilobytes\n"), (long long) (N_EQUATIONS + 3LL) * n_tokens * sizeof(token_type) * 2LL / 1000LL);
+	fprintf(gfp, _("\nMaximum memory usage: %ld kilobytes\n"), ((long) (N_EQUATIONS + 3L) * n_tokens * sizeof(token_type) * 2L) / 1000L);
 	return true;
 }
 
@@ -88,28 +91,87 @@ int
 solve_cmd(cp)
 char	*cp;
 {
-	int	i;
-	char	buf[MAX_CMD_LEN];
+	int		i, j;
+	char		buf[MAX_CMD_LEN];
+	long		v;	/* variable */
+	token_type	want;
+	int		diff_sign;
+	int		verify_flag = false, plural_flag = false, once_through;
 
 	if (cur_equation < 0 || cur_equation >= n_equations || n_lhs[cur_equation] <= 0 || n_rhs[cur_equation] <= 0) {
 		error(_("No current equation."));
 		return false;
 	}
 	i = next_espace();
+	if (strcmp_tospace(cp, "verify") == 0) {
+		verify_flag = true;
+		cp = skip_param(cp);
+	}
 	if (*cp == '\0') {
-		my_strlcpy(prompt_str, _("Enter variable or 0: "), sizeof(prompt_str));
+		my_strlcpy(prompt_str, _("Enter variable to solve for: "), sizeof(prompt_str));
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
 	}
-	if (parse_equation(i, cp)) {
-		if (n_lhs[i] && n_rhs[i]) {
-			error(_("Invalid argument."));
+	if (verify_flag) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL || extra_characters(cp)) {
+			return false;
+		}
+		copy_espace(cur_equation, i);
+		want.level = 1;
+		want.kind = VARIABLE;
+		want.token.variable = v;
+		if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) > 0) {
+			return_result(cur_equation);
+			if (compare_es(cur_equation, i)) {
+				warning(_("Nothing was done, so no need to verify."));
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+				return true;
+			}
+			for (j = 0; j < n_rhs[cur_equation]; j += 2) {
+				if (rhs[cur_equation][j].kind == VARIABLE && (rhs[cur_equation][j].token.variable & VAR_MASK) == SIGN) {
+					plural_flag = true;
+					break;
+				}
+			}
+			subst_var_with_exp(lhs[i], &n_lhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
+			subst_var_with_exp(rhs[i], &n_rhs[i], rhs[cur_equation], n_rhs[cur_equation], v);
+			calc_simp(lhs[i], &n_lhs[i]);
+			calc_simp(rhs[i], &n_rhs[i]);
+			once_through = false;
+check_result:
+			if (se_compare(lhs[i], n_lhs[i], rhs[i], n_rhs[i], &diff_sign) && !diff_sign) {
+#if	!SILENT
+				if (plural_flag)
+					printf(_("Solutions verified.\n"));
+				else
+					printf(_("Solution verified.\n"));
+#endif
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+				return true;
+			} else {
+				if (!once_through) {
+					symb_flag = true;
+					simpa_side(lhs[i], &n_lhs[i], true, false);
+					simpa_side(rhs[i], &n_rhs[i], true, false);
+					symb_flag = false;
+					once_through = true;
+					goto check_result;
+				}
+				error(_("Solution may be incorrect."));
+				return false;
+			}
 		} else {
-			if (solve(i, cur_equation)) {
+#if	!SILENT
+			printf(_("Solve failed.\n"));
+#endif
+		}
+	} else {
+		if (parse_equation(i, cp)) {
+			if (solve_espace(i, cur_equation)) {
 				return return_result(cur_equation);
 			}
 		}
@@ -148,7 +210,7 @@ char	*cp;		/* the command line */
 int	product_flag;	/* true for product, otherwise sum */
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;			/* variable */
 	double		start, end, step = 1.0;
 	int		result_en;
 	int		n, ns;
@@ -352,7 +414,7 @@ int		*np;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;	/* found sub-expression flags */
-	long	v;
+	long	v;			/* variable */
 	char	temp_buf[50];
 
 	if (*np <= 0) {
@@ -535,13 +597,19 @@ char	*cp;
 {
 	int	i, j;
 
+	if (!readline_enabled) {
+		error(_("Readline is disabled."));
+		return false;
+	}
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
 	for (; i <= j; i++) {
-		if (!push_en(i)) {
-			error(_("Push failed."));
-			return false;
+		if (n_lhs[i]) {
+			if (!push_en(i)) {
+				error(_("Push failed."));
+				return false;
+			}
 		}
 	}
 	debug_string(0, _("Expression(s) pushed.  Press the UP key to access."));
@@ -557,14 +625,16 @@ int
 push_en(en)
 int	en;	/* equation space number */
 {
-	if (readline_enabled) {
-		high_prec = true;
-		add_history(list_equation(en, false));
-		high_prec = false;
-		return true;
-	} else {
+	char	*cp;
+
+	high_prec = true;
+	cp = list_equation(en, false);
+	high_prec = false;
+	if (cp == NULL)
 		return false;
-	}
+	add_history(cp);
+	last_history_string = cp;
+	return true;
 }
 #endif
 
@@ -649,7 +719,7 @@ output_options()
 	}
 	fprintf(gfp, "right_associative_power\n");
 
-	fprintf(gfp, "special_variable_character = %c\n", special_variable_character);
+	fprintf(gfp, "special_variable_characters = %s\n", special_variable_characters);
 }
 
 /*
@@ -723,19 +793,15 @@ char	*cp;
 			cp = cp1;
 			debug_level = i;
 		}
-		return true;
+		goto check_return;
 	}
 #endif
 	if (strncasecmp(cp, "special", 7) == 0) {
 		cp = skip_param(cp);
 		if (negate) {
-			special_variable_character = '\0';
+			special_variable_characters[0] = '\0';
 		} else {
-			if (*cp == '\0' || cp[1]) {
-				error(_("Please specify a single character."));
-				return false;
-			}
-			special_variable_character = *cp;
+			my_strlcpy(special_variable_characters, cp, sizeof(special_variable_characters));
 		}
 		return true;
 	}
@@ -751,49 +817,49 @@ char	*cp;
 			cp = cp1;
 			screen_columns = i;
 		}
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "autosolve", 9) == 0) {
 		cp = skip_param(cp);
 		autosolve = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "autocalc", 8) == 0) {
 		cp = skip_param(cp);
 		autocalc = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "display2d", 7) == 0) {
 		cp = skip_param(cp);
 		display2d = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
 		cp = skip_param(cp);
 		preserve_roots = !negate;
-		return true;
+		goto check_return;
 	}
-	if (strncasecmp(cp, "true_modulus", 8) == 0) {
+	if (strncasecmp(cp, "true_modulus", 10) == 0) {
 		cp = skip_param(cp);
 		true_modulus = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
 		cp = skip_param(cp);
 		color_flag = !negate;
 		cur_color = -1;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "bold", 4) == 0) {
 		cp = skip_param(cp);
 		bold_colors = !negate;
 		cur_color = -1;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "finance", 7) == 0) {
 		cp = skip_param(cp);
@@ -816,20 +882,25 @@ char	*cp;
 			cp = cp1;
 			finance_option = i;
 		}
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "factor_integers", 6) == 0) {
 		cp = skip_param(cp);
 		factor_int_flag = !negate;
-		return true;
+		goto check_return;
 	}
 	if (strncasecmp(cp, "right_associative_power", 5) == 0) {
 		cp = skip_param(cp);
 		right_associative_power = !negate;
-		return true;
+		goto check_return;
 	}
 	error(_("Unknown set option."));
 	return false;
+
+check_return:
+	if (extra_characters(cp))
+		return false;
+	return true;
 }
 
 /*
@@ -873,7 +944,7 @@ char	*cp;
 	char	*cp1;
 	char	buf[MAX_CMD_LEN];
 
-	if (test_mode) {
+	if (test_mode || !isatty(0)) {
 		return true;
 	}
 	if (*cp == '\0') {
@@ -918,7 +989,9 @@ char	*cp;
 		if (exists[i1]) {
 			k = next_espace();
 			copy_espace(i1, k);
-			return_result(k);
+			if (!return_result(k)) {
+				return false;
+			}
 		}
 	}
 	return true;
@@ -938,7 +1011,7 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	int		has_imag, has_real;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
-	long		v;
+	long		v;			/* variable */
 
 	if (current_not_defined()) {
 		return false;
@@ -1001,6 +1074,9 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 		error(_("Failed, expression is not a mix."));
 		return false;
 	}
+	do {
+		simp_loop(dest, &n1);
+	} while (factor_plus(dest, &n1, v, 0.0));
 	simp_divide(dest, &n1);
 	if (n_rhs[i]) {
 		blt(lhs[j], lhs[i], n_lhs[i] * sizeof(token_type));
@@ -1119,11 +1195,10 @@ int
 calculate_cmd(cp)
 char	*cp;
 {
-	int		i, j, k;
-	long		v, last_v, it_v = 0;
+	int		i, k1, k;
+	long		v, last_v, it_v = 0;	/* Mathomatic variables */
 	long		counter, counter_max;
-	sign_array_type	sa_mark;
-	sign_array_type	sa_value;
+	sign_array_type	sa_mark, sa_value;
 	long		l, iterations = 1;
 	token_type	*source;
 	int		n;
@@ -1169,7 +1244,7 @@ char	*cp;
 		}
 		iterations = decstrtol(cp, &cp);
 		if (*cp || iterations <= 0) {
-			error(_("Invalid number of iterations."));
+			error(_("Positive integer required."));
 			return false;
 		}
 	}
@@ -1180,11 +1255,11 @@ calc_again:
 	last_v = 0;
 	for (;;) {
 		v = -1;
-		for (j = 0; j < n; j += 2) {
-			if (source[j].kind == VARIABLE) {
-				if (source[j].token.variable > last_v
-				    && (v == -1 || source[j].token.variable < v))
-					v = source[j].token.variable;
+		for (k1 = 0; k1 < n; k1 += 2) {
+			if (source[k1].kind == VARIABLE) {
+				if (source[k1].token.variable > last_v
+				    && (v == -1 || source[k1].token.variable < v))
+					v = source[k1].token.variable;
 			}
 		}
 		if (v == -1)
@@ -1196,29 +1271,26 @@ calc_again:
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
 		if (!get_expr(tlhs, &n_tlhs)) {
-			if (repeat_flag) {
-				return true;
-			}
 			continue;
 		}
 		value_entered = true;
-		/* disguise all variables in the entered expression by making them negative: */
-		for (j = 0; j < n_tlhs; j += 2)
-			if (tlhs[j].kind == VARIABLE)
-				tlhs[j].token.variable = -tlhs[j].token.variable;
+		/* Disguise all variables in the entered expression by making them negative. */
+		/* That way they won't be improperly substituted. */
+		for (k1 = 0; k1 < n_tlhs; k1 += 2)
+			if (tlhs[k1].kind == VARIABLE)
+				tlhs[k1].token.variable = -tlhs[k1].token.variable;
 		subst_var_with_exp(trhs, &n_trhs, tlhs, n_tlhs, v);
 	}
-	/* restore disguised variables: */
-	for (j = 0; j < n_trhs; j += 2)
-		if (trhs[j].kind == VARIABLE && trhs[j].token.variable < 0)
-			trhs[j].token.variable = -trhs[j].token.variable;
+	/* Restore disguised variables: */
+	for (k1 = 0; k1 < n_trhs; k1 += 2)
+		if (trhs[k1].kind == VARIABLE && trhs[k1].token.variable < 0)
+			trhs[k1].token.variable = -trhs[k1].token.variable;
 	if (it_v) {
+		/* Handle the iteration option, where the simplified result is repeatedly plugged into variable it_v. */
 		list_var(it_v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
 		while (!get_expr(tes, &n_tes)) {
-			if (repeat_flag) {
-				return true;
-			}
+			return repeat_flag;
 		}
 		value_entered = true;
 		calc_simp(tes, &n_tes);
@@ -1238,50 +1310,54 @@ calc_again:
 		}
 	}
 	calc_simp(trhs, &n_trhs);
-	fprintf(gfp, "\n");
-	for (j = 0; j < ARR_CNT(sa_mark); j++)
-		sa_mark[j] = false;
-	for (j = 0; j < n_trhs; j += 2) {
-		if (trhs[j].kind == VARIABLE && (trhs[j].token.variable & VAR_MASK) == SIGN) {
-			sa_mark[(trhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+
+	/* Now substitute all sign variables with +1 and -1. */
+	CLEAR_ARRAY(sa_mark);
+	for (k1 = 0; k1 < n_trhs; k1 += 2) {
+		if (trhs[k1].kind == VARIABLE && (trhs[k1].token.variable & VAR_MASK) == SIGN) {
+			sa_mark[(trhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 		}
 	}
-	for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
-		if (sa_mark[j]) {
+	for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+		if (sa_mark[k1]) {
 			k++;
 		}
 	}
 	counter_max = (1L << k) - 1L;
+	if (counter_max) {
+		fprintf(gfp, _("There are %ld solutions.\n"), counter_max + 1);
+	}
+	fprintf(gfp, "\n");
 	for (counter = 0; counter <= counter_max; counter++) {
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
-		for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
-			if (sa_mark[j]) {
-				sa_value[j] = (((1L << k) & counter) != 0);
+		for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+			if (sa_mark[k1]) {
+				sa_value[k1] = (((1L << k) & counter) != 0);
 				k++;
 			}
 		}
-		for (j = 0; j < n_tlhs; j += 2) {
-			if (tlhs[j].kind == VARIABLE && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
-				if (sa_value[(tlhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
-					tlhs[j].kind = CONSTANT;
-					tlhs[j].token.constant = -1.0;
+		for (k1 = 0; k1 < n_tlhs; k1 += 2) {
+			if (tlhs[k1].kind == VARIABLE && (tlhs[k1].token.variable & VAR_MASK) == SIGN) {
+				if (sa_value[(tlhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+					tlhs[k1].kind = CONSTANT;
+					tlhs[k1].token.constant = -1.0;
 				} else {
-					tlhs[j].kind = CONSTANT;
-					tlhs[j].token.constant = 1.0;
+					tlhs[k1].kind = CONSTANT;
+					tlhs[k1].token.constant = 1.0;
 				}
 			}
 		}
-		for (j = 0, k = false; j < ARR_CNT(sa_mark); j++) {
-			if (sa_mark[j]) {
+		for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
+			if (sa_mark[k1]) {
 				if (k) {
 					fprintf(gfp, ", ");
 				} else {
-					fprintf(gfp, _("Solution #%ld with "), counter + 1);
+					fprintf(gfp, _("Solution number %ld with "), counter + 1);
 				}
-				list_var((long) SIGN + (((long) j) << VAR_SHIFT), 0);
+				list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
 				fprintf(gfp, "%s = ", var_str);
-				if (sa_value[j]) {
+				if (sa_value[k1]) {
 					fprintf(gfp, "-1");
 				} else {
 					fprintf(gfp, "1");
@@ -1353,28 +1429,43 @@ int	*diff_signp;
 		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
-	sign_flag = true;
+	sign_cmp_flag = true;
 	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], diff_signp);
-	sign_flag = false;
+	sign_cmp_flag = false;
 	return rv;
 }
 
-static int
-compare_lhs(i, j)
-int	i, j;
+/*
+ * Compare two equation spaces.
+ *
+ * Return true if identical.
+ */
+int
+compare_es(i, j)
+int	i, j;	/* equation space numbers */
 {
 	int	rv;
 	int	diff_sign;
 
-	rv = se_compare(lhs[i], n_lhs[i], lhs[i], n_lhs[i], &diff_sign);
-	if (!rv || diff_sign) {
-		error(_("Error in compare function or too many terms to compare."));
+	if (n_lhs[i] == 0 || n_lhs[j] == 0)
 		return false;
-	}
-	sign_flag = true;
+	/* compare the two left hand sides */
+	sign_cmp_flag = true;
 	rv = se_compare(lhs[i], n_lhs[i], lhs[j], n_lhs[j], &diff_sign);
-	sign_flag = false;
-	return(rv && !diff_sign);
+	sign_cmp_flag = false;
+	if (!rv || diff_sign)
+		return false;
+	if (n_rhs[i] == 0 && n_rhs[j] == 0)
+		return true;	/* two expressions, not equations */
+	if (n_rhs[i] == 0 || n_rhs[j] == 0)
+		return false;	/* mix of expression and equation */
+	/* compare the two right hand sides */
+	sign_cmp_flag = true;
+	rv = se_compare(rhs[i], n_rhs[i], rhs[j], n_rhs[j], &diff_sign);
+	sign_cmp_flag = false;
+	if (!rv || diff_sign)
+		return false;
+	return true;
 }
 
 /*
@@ -1387,6 +1478,9 @@ char	*cp;
 	int		i, j;
 	int		diff_sign;
 
+	if (strcmp_tospace(cp, "with") == 0) {
+		cp = skip_param(cp);
+	}
 	i = decstrtol(cp, &cp) - 1;
 	if (not_defined(i)) {
 		return false;
@@ -1406,7 +1500,7 @@ char	*cp;
 		if (n_rhs[i] == 0 && n_rhs[j] == 0) {
 			simp_loop(lhs[i], &n_lhs[i]);
 			simp_loop(lhs[j], &n_lhs[j]);
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
@@ -1419,13 +1513,13 @@ char	*cp;
 				list_sub(j);
 			}
 #endif
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
 			uf_simp(lhs[i], &n_lhs[i]);
 			uf_simp(lhs[j], &n_lhs[j]);
-			if (compare_lhs(i, j)) {
+			if (compare_es(i, j)) {
 				fprintf(gfp, _("Expressions are identical.\n"));
 				return true;
 			}
@@ -1435,6 +1529,10 @@ char	*cp;
 		error(_("Cannot compare an expression with an equation."));
 		return false;
 	}
+	if (compare_es(i, j)) {
+		fprintf(gfp, _("Equations are identical.\n"));
+		return true;
+	}
 	if (solved_equation(i) && solved_equation(j)) {
 		simp_loop(rhs[i], &n_rhs[i]);
 		simp_loop(rhs[j], &n_rhs[j]);
@@ -1460,11 +1558,10 @@ char	*cp;
 		}
 	}
 	debug_string(0, _("Solving both equations for zero and unfactoring..."));
-	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
-	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
-		error(_("Can't solve for zero!"));
+	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0)
+		return false;
+	if (solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0)
 		return false;
-	}
 	uf_simp(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
@@ -1478,11 +1575,10 @@ char	*cp;
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
-	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
-		error(_("Can't solve for zero!"));
+	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0)
+		return false;
+	if (solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0)
 		return false;
-	}
 	uf_simp(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
@@ -1515,13 +1611,12 @@ int
 divide_cmd(cp)
 char	*cp;
 {
-	long		v, v_tmp;
+	long		v = 0, v_tmp;		/* Mathomatic variables */
 	int		i, j;
 	int		nl, nr;
-	double		d1, d2, d3, d4;
+	double		d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
 
-	v = 0;
 	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
@@ -1540,39 +1635,51 @@ char	*cp;
 		return false;
 	}
 	fprintf(gfp, "\n");
-	calc_simp(lhs[i], &nl);
+/* simplify the input expressions */
 	calc_simp(rhs[i], &nr);
+	calc_simp(lhs[i], &nl);
+/* if division by zero, display a warning */
+	if (get_constant(lhs[i], nl, &d2)) {
+		check_divide_by_zero(d2);
+	}
+/* Do constant division if 2 normal numbers were entered */
 	if (get_constant(rhs[i], nr, &d1) && get_constant(lhs[i], nl, &d2)) {
-		d4 = modf(d1 / d2, &d3);
-		fprintf(gfp, _("Result of numerical division: %.*g/%.*g = %.*g\n"), precision, d1, precision, d2, precision, d1 / d2);
-		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d3, precision, d4 * d2);
+		d3 = gcd_verified(d1, d2);
+		d4 = modf(d1 / d2, &d5);
+		if (d3 == 0 || d3 == 1) {
+			fprintf(gfp, "%.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d2);
+		} else {
+			fprintf(gfp, "%.*g/%.*g = %.*g/%.*g = %.*g\n", precision, d1, precision, d2, precision, d1 / d3, precision, d2 / d3, precision, d1 / d2);
+		}
+		fprintf(gfp, _("Quotient: %.*g, Remainder: %.*g\n"), precision, d5, precision, d4 * d2);
 		d1 = fabs(d1);
 		d2 = fabs(d2);
-		d3 = gcd_verified(d1, d2);
-		if (d3 <= 0.0) {
+		if (d3 == 0.0) {
 			fprintf(gfp, _("No GCD found.\n"));
 			return true;
 		}
-		if (factor_int_flag && factor_one(d3)) {
-			fprintf(gfp, "GCD = ");
+		fprintf(gfp, "GCD = ");
+		if (factor_one(d3)) {
 			display_unique();
 		} else {
-			fprintf(gfp, "GCD = %.*g\n", precision, d3);
+			fprintf(gfp, "%.*g\n", precision, d3);
 		}
-		if (factor_int_flag && factor_one((d1 * d2) / d3)) {
-			fprintf(gfp, "LCM = ");
+		fprintf(gfp, "LCM = ");
+		if (factor_one((d1 * d2) / d3)) {
 			display_unique();
 		} else {
-			fprintf(gfp, "LCM = %.*g\n", precision, (d1 * d2) / d3);
+			fprintf(gfp, "%.*g\n", precision, (d1 * d2) / d3);
 		}
 		return true;
 	}
+/* else do complex number division if 2 complex numbers were entered */
 	if (parse_complex(rhs[i], nr, &c1) && parse_complex(lhs[i], nl, &c2)) {
 		c3 = complex_div(c1, c2);
 		fprintf(gfp, _("Result of complex number division:\n"));
 		fprintf(gfp, "%.*g %+.*g*i\n\n", precision, c3.re, precision, c3.im);
 		return true;
 	}
+/* else do polynomial division and univariate GCD display */
 	v_tmp = v;
 	if (poly_div(rhs[i], nr, lhs[i], nl, &v_tmp)) {
 		simp_divide(tlhs, &n_tlhs);
@@ -1596,7 +1703,7 @@ char	*cp;
 	}
 	if (j) {
 		simp_divide(trhs, &n_trhs);
-		fprintf(gfp, _("Polynomial GCD (Euclidean algorithm iterations = %d):\n"), j);
+		fprintf(gfp, _("Polynomial GCD (iterations = %d):\n"), j);
 		display_fractions_and_group(trhs, &n_trhs);
 		list_factor(trhs, &n_trhs, false);
 		fprintf(gfp, "\n");
@@ -1614,19 +1721,18 @@ int
 eliminate_cmd(cp)
 char	*cp;
 {
-	long	v, last_v, v1, va[MAX_VARS];
-	int	i, n;
-	int	success_flag = false, did_something = false, repeat_flag = false;
+	long	v, last_v, v1, va[MAX_VARS];		/* Mathomatic variables */
+	int	vc = 0;					/* variable count */
+	int	i = 0, n;
+	int	success_flag = false, did_something = false, repeat_flag = false, using_flag;
 	char	used[N_EQUATIONS];
-	int	vc = 0;
 	char	*cp_start;
 
 	if (*cp == '\0') {
 		error(_("Please specify the variables to eliminate or \"all\" for all variables."));
 		return false;
 	}
-	for (i = 0; i < ARR_CNT(used); i++)
-		used[i] = false;
+	CLEAR_ARRAY(used);
 	if (current_not_defined()) {
 		return false;
 	}
@@ -1692,6 +1798,14 @@ next_var:
 		}
 		return did_something;
 	}
+	using_flag = (strcmp_tospace(cp, "using") == 0);
+	if (using_flag) {
+		cp = skip_param(cp);
+		i = decstrtol(cp, &cp) - 1;
+		if (not_defined(i)) {
+			return false;
+		}
+	}
 	if (!var_in_equation(cur_equation, v)) {
 #if	!SILENT
 		if (!repeat_flag) {
@@ -1701,16 +1815,7 @@ next_var:
 #endif
 		goto next_var;
 	}
-	if (strcmp_tospace(cp, "using") == 0) {
-		cp = skip_param(cp);
-		i = decstrtol(cp, &cp) - 1;
-		if (not_defined(i)) {
-			return false;
-		}
-		if (i == cur_equation) {
-			error(_("Error: source and destination are the same."));
-			return false;
-		}
+	if (using_flag) {
 		if (!elim_sub(i, v))
 			goto next_var;
 	} else {
@@ -1740,11 +1845,15 @@ next_var:
 
 static int
 elim_sub(i, v)
-int	i;
-long	v;
+int	i;	/* equation number */
+long	v;	/* variable */
 {
 	token_type	want;
 
+	if (i == cur_equation) {
+		error(_("Error: source and destination are the same."));
+		return false;
+	}
 #if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
@@ -1923,8 +2032,8 @@ char	*cp;
 {
 	int		i, j, k;
 	int		i1;
-	int		vc, cnt;
-	long		v1, last_v;
+	long		v1, last_v;		/* Mathomatic variables */
+	int		vc, cnt;		/* variable counts */
 	sort_type	va[MAX_VARS];
 	token_type	*p1;
 	int		n1;
@@ -2035,7 +2144,9 @@ char	*cp;
 				approximate_roots = true;
 				simp_sub(i);
 				approximate_roots = false;
-				return_result(i);
+				if (!return_result(i)) {
+					return false;
+				}
 			}
 		}
 	} while (*cp);
@@ -2051,10 +2162,10 @@ char	*cp;
 {
 	int	i, j;
 	int	n;
-	long	last_v, v;
+	long	last_v, v;	/* Mathomatic variables */
 	char	*cp_start;
-	long	va[MAX_VARS];
-	int	vc;
+	long	va[MAX_VARS];	/* variable array */
+	int	vc;		/* variable count */
 	int	found;
 	char	*cp1;
 
@@ -2122,14 +2233,9 @@ char	*cp;
 				}
 				cp1 = skip_param(cp);
 				input_column += (cp1 - cp_start);
-				if (!case_sensitive_flag) {
-					str_tolower(cp1);
-				}
-				if ((cp1 = parse_section(scratch, &n, cp1)) == NULL || n <= 0) {
+				if ((cp1 = parse_expr(scratch, &n, cp1)) == NULL || n <= 0) {
 					return false;
 				}
-				if (extra_characters(cp1))
-					return false;
 				goto do_this;
 			}
 		}
@@ -2139,7 +2245,8 @@ char	*cp;
 			continue;
 		}
 do_this:
-		/* disguise all variables in the entered expression by making them negative: */
+		/* Disguise all variables in the entered expression by making them negative. */
+		/* That way they won't be improperly substituted. */
 		for (j = 0; j < n; j += 2) {
 			if (scratch[j].kind == VARIABLE) {
 				scratch[j].token.variable = -scratch[j].token.variable;
@@ -2148,7 +2255,7 @@ do_this:
 		subst_var_with_exp(tlhs, &n_tlhs, scratch, n, v);
 		subst_var_with_exp(trhs, &n_trhs, scratch, n, v);
 	}
-	/* restore disguised variables: */
+	/* Restore disguised variables: */
 	for (j = 0; j < n_tlhs; j += 2)
 		if (tlhs[j].kind == VARIABLE && tlhs[j].token.variable < 0)
 			tlhs[j].token.variable = -tlhs[j].token.variable;
@@ -2170,10 +2277,17 @@ int
 simplify_cmd(cp)
 char	*cp;
 {
-	int	i, j;
-	int	quick_flag = false, symb = false, frac_flag = false;
+	int		i, i1, j;
+	int		k, k1;
+	long		counter, counter_max, previous_solution_number[N_EQUATIONS];
+	sign_array_type	sa_mark, sa_value;
+	int		sign_flag = false, quick_flag = false, symb = false, frac_flag = false;
 
 	for (;; cp = skip_param(cp)) {
+		if (strncasecmp(cp, "sign", 4) == 0) {
+			sign_flag = true;
+			continue;
+		}
 		if (strncasecmp(cp, "symbolic", 4) == 0) {
 			symb = true;
 			continue;
@@ -2196,7 +2310,100 @@ char	*cp;
 		if (n_lhs[i]) {
 			simpa_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
 			simpa_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
-			return_result(i);
+			if (!return_result(i)) {
+				symb_flag = false;
+				return false;
+			}
+			if (!sign_flag)
+				continue;
+			/* Now substitute all sign variables with +1 and -1. */
+			CLEAR_ARRAY(previous_solution_number);
+			CLEAR_ARRAY(sa_mark);
+			for (k1 = 0; k1 < n_lhs[i]; k1 += 2) {
+				if (lhs[i][k1].kind == VARIABLE && (lhs[i][k1].token.variable & VAR_MASK) == SIGN) {
+					sa_mark[(lhs[i][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+				}
+			}
+			for (k1 = 0; k1 < n_rhs[i]; k1 += 2) {
+				if (rhs[i][k1].kind == VARIABLE && (rhs[i][k1].token.variable & VAR_MASK) == SIGN) {
+					sa_mark[(rhs[i][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+				}
+			}
+			for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+				if (sa_mark[k1]) {
+					k++;
+				}
+			}
+			if (k == 0)
+				continue;
+			counter_max = (1L << k) - 1L;
+			for (counter = 0; counter <= counter_max; counter++) {
+				i1 = next_espace();
+				copy_espace(i, i1);
+				for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+					if (sa_mark[k1]) {
+						sa_value[k1] = (((1L << k) & counter) != 0);
+						k++;
+					}
+				}
+				for (k1 = 0; k1 < n_lhs[i1]; k1 += 2) {
+					if (lhs[i1][k1].kind == VARIABLE && (lhs[i1][k1].token.variable & VAR_MASK) == SIGN) {
+						if (sa_value[(lhs[i1][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+							lhs[i1][k1].kind = CONSTANT;
+							lhs[i1][k1].token.constant = -1.0;
+						} else {
+							lhs[i1][k1].kind = CONSTANT;
+							lhs[i1][k1].token.constant = 1.0;
+						}
+					}
+				}
+				for (k1 = 0; k1 < n_rhs[i1]; k1 += 2) {
+					if (rhs[i1][k1].kind == VARIABLE && (rhs[i1][k1].token.variable & VAR_MASK) == SIGN) {
+						if (sa_value[(rhs[i1][k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+							rhs[i1][k1].kind = CONSTANT;
+							rhs[i1][k1].token.constant = -1.0;
+						} else {
+							rhs[i1][k1].kind = CONSTANT;
+							rhs[i1][k1].token.constant = 1.0;
+						}
+					}
+				}
+				for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
+					if (sa_mark[k1]) {
+						if (k) {
+							fprintf(gfp, ", ");
+						} else {
+							fprintf(gfp, _("Solution number %ld with "), counter + 1);
+						}
+						list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
+						fprintf(gfp, "%s = ", var_str);
+						if (sa_value[k1]) {
+							fprintf(gfp, "-1");
+						} else {
+							fprintf(gfp, "1");
+						}
+						k = true;
+					}
+				}
+				if (k)
+					fprintf(gfp, ":\n");
+				simpa_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
+				simpa_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+				for (k1 = 0; k1 < ARR_CNT(previous_solution_number); k1++) {
+					if (previous_solution_number[k1]) {
+						if (compare_es(k1, i1)) {
+							n_lhs[i1] = 0;
+							n_rhs[i1] = 0;
+							fprintf(gfp, _("is identical to solution number %ld.\n"), previous_solution_number[k1]);
+							break;
+						}
+					}
+				}
+				if (n_lhs[i1]) {
+					list_sub(i1);
+					previous_solution_number[i1] = counter + 1;
+				}
+			}
 		}
 	}
 	symb_flag = false;
@@ -2211,23 +2418,26 @@ factor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	i1, count;
-	long	v;
+	int	i1;
+	long	v;				/* variable */
 	double	d;
-	char	*cp_start, buf[MAX_CMD_LEN];
-	int	valid_range = false;
-	int	repeat_flag = false;
+	char	buf[MAX_CMD_LEN];
+	int	valid_range = false, repeat_flag = false, power_flag = false;
 
-	cp_start = cp;
 	if (strcmp_tospace(cp, "number") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "numbers") == 0) {
 		repeat_flag = true;
 		cp = skip_param(cp);
 	} else {
+		if (strcmp_tospace(cp, "power") == 0) {
+			power_flag = true;
+			cp = skip_param(cp);
+		}
 		valid_range = get_range(&cp, &i, &j);
 		if (!valid_range) {
-			cp = cp_start;
+			if (*cp == '\0' || power_flag)
+				return false;
 		}
 	}
 	if (!valid_range) {
@@ -2241,11 +2451,11 @@ char	*cp;
 			if (*cp == '\0') {
 				return true;
 			}
-			for (count = 1; *cp; count++) {
+			for (; *cp; ) {
 				d = strtod(cp, &cp);
 				cp = skip_space(cp);
 				if (!factor_one(d)) {
-					error(_("Not a valid integer or number too large."));
+					error(_("Not a valid integer or number too large to factor."));
 					return false;
 				}
 				display_unique();
@@ -2253,22 +2463,46 @@ char	*cp;
 		} while (repeat_flag);
 		return true;
 	}
-	v = 0;
-	do {
-		if (*cp) {
-			if ((cp = parse_var2(&v, cp)) == NULL) {
-				return false;
-			}
-		}
+	if (power_flag) {
+		if (extra_characters(cp))
+			return false;
 		for (i1 = i; i1 <= j; i1++) {
 			if (n_lhs[i1]) {
-				simpv_side(lhs[i1], &n_lhs[i1], v);
-				simpv_side(rhs[i1], &n_rhs[i1], v);
+				factor_power(lhs[i1], &n_lhs[i1]);
+				do {
+					simp_loop(lhs[i1], &n_lhs[i1]);
+				} while (factor_power(lhs[i1], &n_lhs[i1]));
+				if (n_rhs[i1]) {
+					factor_power(rhs[i1], &n_rhs[i1]);
+					do {
+						simp_loop(rhs[i1], &n_rhs[i1]);
+					} while (factor_power(rhs[i1], &n_rhs[i1]));
+				}
+				if (!return_result(i1))
+					return false;
+			}
+		}
+	} else {
+		v = 0;
+		do {
+			if (*cp) {
+				if ((cp = parse_var2(&v, cp)) == NULL) {
+					return false;
+				}
+			}
+			for (i1 = i; i1 <= j; i1++) {
+				if (n_lhs[i1]) {
+					simpv_side(lhs[i1], &n_lhs[i1], v);
+					simpv_side(rhs[i1], &n_rhs[i1], v);
+				}
+			}
+		} while (*cp);
+		for (i1 = i; i1 <= j; i1++) {
+			if (n_lhs[i1]) {
+				if (!return_result(i1))
+					return false;
 			}
 		}
-	} while (*cp);
-	for (i1 = i; i1 <= j; i1++) {
-		return_result(i1);
 	}
 	return true;
 }
@@ -2281,24 +2515,71 @@ unfactor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	fully_flag;
+	int	quick_flag = false, fully_flag = false, power_flag = false;
 
-	fully_flag = (strncasecmp(cp, "fully", 4) == 0);
-	if (fully_flag) {
-		cp = skip_param(cp);
+	for (;; cp = skip_param(cp)) {
+		if (strncasecmp(cp, "quick", 5) == 0) {
+			quick_flag = true;
+			continue;
+		}
+		if (strncasecmp(cp, "fully", 4) == 0) {
+			fully_flag = true;
+			continue;
+		}
+		if (strncasecmp(cp, "power", 5) == 0) {
+			power_flag = true;
+			continue;
+		}
+		break;
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
 	}
-	partial_flag = !fully_flag;
-	for (; i <= j; i++) {
-		if (n_lhs[i]) {
-			uf_simp(lhs[i], &n_lhs[i]);
-			uf_simp(rhs[i], &n_rhs[i]);
-			return_result(i);
+	if (power_flag) {
+		for (; i <= j; i++) {
+			if (n_lhs[i]) {
+				if (fully_flag) {
+					uf_allpower(lhs[i], &n_lhs[i]);
+				} else {
+					uf_power(lhs[i], &n_lhs[i]);
+				}
+				elim_loop(lhs[i], &n_lhs[i]);
+				if (n_rhs[i]) {
+					if (fully_flag) {
+						uf_allpower(rhs[i], &n_rhs[i]);
+					} else {
+						uf_power(rhs[i], &n_rhs[i]);
+					}
+					elim_loop(rhs[i], &n_rhs[i]);
+				}
+				if (!return_result(i)) {
+					return false;
+				}
+			}
 		}
+	} else {
+		partial_flag = !fully_flag;
+		for (; i <= j; i++) {
+			if (n_lhs[i]) {
+				if (quick_flag) {
+					uf_tsimp(lhs[i], &n_lhs[i]);
+					if (n_rhs[i]) {
+						uf_tsimp(rhs[i], &n_rhs[i]);
+					}
+				} else {
+					uf_simp(lhs[i], &n_lhs[i]);
+					if (n_rhs[i]) {
+						uf_simp(rhs[i], &n_rhs[i]);
+					}
+				}
+				if (!return_result(i)) {
+					partial_flag = true;
+					return false;
+				}
+			}
+		}
+		partial_flag = true;
 	}
-	partial_flag = true;
 	return true;
 }
 
@@ -2318,7 +2599,8 @@ char	*cp;
 		if (n_lhs[i]) {
 			frac_side(lhs[i], &n_lhs[i]);
 			frac_side(rhs[i], &n_rhs[i]);
-			return_result(i);
+			if (!return_result(i))
+				return false;
 		}
 	}
 	return true;
@@ -2332,10 +2614,15 @@ int
 quit_cmd(cp)
 char	*cp;
 {
-	if (extra_characters(cp))
-		return false;
-	exit_program(0);
-	return false;
+	int	ev = 0;
+
+	if (*cp) {
+		ev = decstrtol(cp, &cp);
+		if (extra_characters(cp))
+			return false;
+	}
+	exit_program(ev);
+	return false;		/* to placate the C compiler */
 }
 #endif
 
@@ -2368,11 +2655,13 @@ char	*cp;
 		cp = buf;
 	}
 	rv = read_sub(fp);
+#if	!SILENT
 	if (rv) {
 		printf(_("Read operation aborted.\n"));
 	} else {
 		printf(_("Finished reading file \"%s\".\n"), cp);
 	}
+#endif
 	fclose(fp);
 	return(!rv);
 }
@@ -2398,11 +2687,16 @@ FILE	*fp;
 		}
 	} else {
 		while ((cp = fgets((char *) tlhs, n_tokens * sizeof(token_type), fp)) != NULL) {
+#if	!SILENT
 			default_color();
 			input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT);
-			printf("%s", cp);
+			if (html_flag)
+				printf("<b>%s</b>", cp);	/* make input bold */
+			else
+				printf("%s", cp);
+#endif
 			if (gfp != stdout) {
-				fprintf(gfp, "%d%s%s", cur_equation + 1, html_flag ? HTML_PROMPT : PROMPT, cp);
+				fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT, cp);
 			}
 			set_error_level(cp);
 			if (!process(cp)) {
@@ -2428,6 +2722,7 @@ char	*cp;
 	int	rv;
 	char	tmp_file[MAX_CMD_LEN];
 
+	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
 #if	CYGWIN
 		my_strlcpy(tmp_file, "mathomatic.tmp", sizeof(tmp_file));
@@ -2506,6 +2801,7 @@ char	*cp;
 	FILE	*fp;
 	int	rv;
 
+	clean_up();	/* end any redirection */
 	if (*cp == '\0') {
 		error(_("No file specified."));
 		return false;
