@@ -1,7 +1,7 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -11,6 +11,7 @@
  *
  * The amplitude is stored in *radiusp and the angle in radians is stored in *thetap.
  */
+void
 rect_to_polar(x, y, radiusp, thetap)
 double	x, y, *radiusp, *thetap;
 {
@@ -77,7 +78,7 @@ char	*cp;
 		radius_root = pow(radius, 1.0 / root);
 	}
 	check_err();
-	fprintf(gfp, _("\nThe polar coordinates are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
+	fprintf(gfp, _("The polar coordinates before root taking are:\n%.12g amplitude and %.12g radians (%.12g degrees).\n\n"),
 	    radius, theta, theta * 180.0 / M_PI);
 	if (root) {
 		if (c.im == 0.0) {
@@ -112,10 +113,11 @@ char	*cp;
 #endif
 
 /*
- * Approximate roots of complex numbers:
+ * Approximate roots of complex numbers in an equation side:
  * (complex^real) and (real^complex) and (complex^complex).
+ * This only gives one root, even when there may be many roots.
  *
- * Returns true if expression was modified.
+ * Return true and display a warning message if the equation side was modified.
  */
 int
 complex_root_simp(equation, np)
@@ -176,46 +178,81 @@ start_over:
 		goto start_over;
 	}
 	if (modified) {
-		debug_string(0, "Warning: complex number root approximated, result may be inaccurate.");
+		warning(_("Complex number root approximated."));
 	}
 	return modified;
 }
 
 /*
- * Get a constant, if the passed expression is a constant.
+ * Get a constant, if the passed expression evaluates to a constant.
+ * This should not be called from low level routines.
  *
- * Return true if successful, with number in "*dp".
+ * Return true if successful, with the floating point constant returned in *dp.
  */
 int
-get_constant(equation, n, dp)
-token_type	*equation;	/* expression pointer */
-int		n;		/* length of expression */
-double		*dp;		/* pointer to returned double */
+get_constant(p1, n, dp)
+token_type	*p1;	/* expression pointer */
+int		n;	/* length of expression */
+double		*dp;	/* pointer to returned double */
 {
-	if (n != 1)
-		return false;
-	switch (equation[0].kind) {
-	case CONSTANT:
-		*dp = equation[0].token.constant;
-		return true;
-	case VARIABLE:
-		if (var_is_const(equation[0].token.variable, dp)) {
+	int	i, j;
+	int	level;
+	double	d1, d2;
+	int	prev_approx_flag;
+
+	if (n == 1) {
+		switch (p1[0].kind) {
+		case CONSTANT:
+			*dp = p1[0].token.constant;
 			return true;
+		case VARIABLE:
+			if (var_is_const(p1[0].token.variable, dp)) {
+				return true;
+			}
 		}
+	} else if (n >= 3) {
+		level = p1[1].level;
+		if (!get_constant(p1, 1, &d1))
+			return false;
+		for (i = 1; i < n; i = j) {
+			if (p1[i].kind != OPERATOR || p1[i].level > level)
+				return false;
+			level = p1[i].level;
+			for (j = i + 2; j < n && p1[j].level > level; j += 2)
+				;
+			if (!get_constant(&p1[i+1], j - (i + 1), &d2))
+				return false;
+			prev_approx_flag = approximate_roots;
+			approximate_roots = true;
+			if (calc(NULL, &d1, p1[i].token.operatr, d2)) {
+				approximate_roots = prev_approx_flag;
+				if (p1[i].token.operatr == POWER && !domain_check)
+					return false;
+				domain_check = false;
+			} else {
+				approximate_roots = prev_approx_flag;
+				return false;
+			}
+		}
+		*dp = d1;
+		return true;
 	}
 	return false;
 }
 
 /*
- * Parse a complex number expression.
+ * Parse a constant complex number expression.
  *
- * If successful return true with complex number in "*cp".
+ * This routine should be fixed to work with multiple constant expressions,
+ * like the above get_constant() does.
+ *
+ * If successful return true with complex number in *cp.
  */
 int
-parse_complex(equation, n, cp)
-token_type	*equation;	/* expression pointer */
-int		n;		/* length of expression */
-complexs	*cp;		/* pointer to returned complex number */
+parse_complex(p1, n, cp)
+token_type	*p1;	/* expression pointer */
+int		n;	/* length of expression */
+complexs	*cp;	/* pointer to returned complex number */
 {
 	int		j;
 	int		imag_cnt = 0, plus_cnt = 0, times_cnt = 0;
@@ -223,7 +260,7 @@ complexs	*cp;		/* pointer to returned complex number */
 	int		level2;
 	double		junk;
 
-	if (get_constant(equation, n, &c.re)) {
+	if (get_constant(p1, n, &c.re)) {
 		c.im = 0.0;
 		*cp = c;
 		return true;
@@ -231,31 +268,31 @@ complexs	*cp;		/* pointer to returned complex number */
 	c.re = 0.0;
 	c.im = 1.0;
 	for (j = n - 1; j >= 0; j--) {
-		switch (equation[j].kind) {
+		switch (p1[j].kind) {
 		case CONSTANT:
 			break;
 		case VARIABLE:
-			if (var_is_const(equation[j].token.variable, &junk))
+			if (var_is_const(p1[j].token.variable, &junk))
 				break;
-			if (equation[j].token.variable != IMAGINARY)
+			if (p1[j].token.variable != IMAGINARY)
 				return false;
 			imag_cnt++;
 			break;
 		case OPERATOR:
-			level2 = equation[j].level;
-			switch (equation[j].token.operatr) {
+			level2 = p1[j].level;
+			switch (p1[j].token.operatr) {
 			case TIMES:
 				if (++times_cnt > 1)
 					return false;
-				if (equation[j-1].level != level2 || equation[j+1].level != level2)
+				if (p1[j-1].level != level2 || p1[j+1].level != level2)
 					return false;
-				if (equation[j-1].kind == VARIABLE && equation[j-1].token.variable == IMAGINARY) {
-					if (!get_constant(&equation[j+1], 1, &c.im))
+				if (p1[j-1].kind == VARIABLE && p1[j-1].token.variable == IMAGINARY) {
+					if (!get_constant(&p1[j+1], 1, &c.im))
 						return false;
 					continue;
 				}
-				if (equation[j+1].kind == VARIABLE && equation[j+1].token.variable == IMAGINARY) {
-					if (!get_constant(&equation[j-1], 1, &c.im))
+				if (p1[j+1].kind == VARIABLE && p1[j+1].token.variable == IMAGINARY) {
+					if (!get_constant(&p1[j-1], 1, &c.im))
 						return false;
 					continue;
 				}
@@ -263,10 +300,10 @@ complexs	*cp;		/* pointer to returned complex number */
 			case DIVIDE:
 				if (++times_cnt > 1)
 					return false;
-				if (equation[j-1].level != level2 || equation[j+1].level != level2)
+				if (p1[j-1].level != level2 || p1[j+1].level != level2)
 					return false;
-				if (equation[j-1].kind == VARIABLE && equation[j-1].token.variable == IMAGINARY) {
-					if (!get_constant(&equation[j+1], 1, &c.im))
+				if (p1[j-1].kind == VARIABLE && p1[j-1].token.variable == IMAGINARY) {
+					if (!get_constant(&p1[j+1], 1, &c.im))
 						return false;
 					c.im = 1.0 / c.im;
 					continue;
@@ -279,11 +316,11 @@ complexs	*cp;		/* pointer to returned complex number */
 			case PLUS:
 				if (++plus_cnt > 1)
 					return false;
-				if (equation[j-1].level == level2 && get_constant(&equation[j-1], 1, &c.re)) {
+				if (p1[j-1].level == level2 && get_constant(&p1[j-1], 1, &c.re)) {
 					continue;
 				}
-				if (equation[j+1].level == level2 && get_constant(&equation[j+1], 1, &c.re)) {
-					if (equation[j].token.operatr == MINUS)
+				if (p1[j+1].level == level2 && get_constant(&p1[j+1], 1, &c.re)) {
+					if (p1[j].token.operatr == MINUS)
 						c.re = -c.re;
 					continue;
 				}
