@@ -41,10 +41,10 @@ long		v;		/* differentiation variable */
  * Symbolically differentiate expression in "equation"
  * (which is a standard equation side) starting at "loc".
  * The current level of parentheses is "level" and
- * do the differentation with respect to variable "v".
+ * do the differentiation with respect to variable "v".
  *
- * Return "true" if successful, "false" if it is beyond this program's
- * capabilities or an error was encountered.
+ * Return true if successful.
+ * Return false if it is beyond this program's capabilities or an error was encountered.
  */
 static int
 d_recurse(equation, np, loc, level, v)
@@ -82,7 +82,7 @@ long		v;
 				break;
 			default:
 /* Oops.  More than one operator on the same level in this expression. */
-				error("Error in d_recurse().");
+				error("Internal error in d_recurse(): differentiating with unparenthesized operators is not allowed.");
 				return false;
 			}
 			op = equation[endloc].token.operatr;
@@ -288,14 +288,19 @@ derivative_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;		/* variable */
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
+	int		simplify_flag = true;
 
 	if (current_not_defined()) {
 		return false;
 	}
+	if (strcmp_tospace(cp, "nosimplify") == 0) {
+		simplify_flag = false;
+		cp = skip_param(cp);
+	}
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
 		source = rhs[cur_equation];
@@ -330,7 +335,7 @@ char	*cp;
 		}
 	}
 	if (no_vars(source, *nps, &v)) {
-		error(_("Current expression contains no variables, result would be zero."));
+		error(_("Current expression contains no variables."));
 		return false;
 	}
 	if (v == 0) {
@@ -339,15 +344,20 @@ char	*cp;
 		}
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		error(_("Variable not found, result would be zero."));
+		error(_("Variable not found; the derivative would be zero."));
 		return false;
 	}
 #if	!SILENT
 	list_var(v, 0);
 	if (n_rhs[cur_equation]) {
-		printf(_("Differentiating the RHS with respect to (%s) and simplifying...\n"), var_str);
+		printf(_("Differentiating the RHS with respect to (%s)"), var_str);
 	} else {
-		printf(_("Differentiating with respect to (%s) and simplifying...\n"), var_str);
+		printf(_("Differentiating with respect to (%s)"), var_str);
+	}
+	if (simplify_flag) {
+		printf(" and simplifying...\n");
+	} else {
+		printf("...\n");
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -358,7 +368,11 @@ char	*cp;
 			error(_("Differentiation failed."));
 			return false;
 		}
-		simpa_side(dest, &n1, true, false);
+		if (simplify_flag) {
+			simpa_side(dest, &n1, true, false);
+		} else {
+			elim_loop(dest, &n1);
+		}
 	}
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
@@ -377,7 +391,7 @@ extrema_cmd(cp)
 char	*cp;
 {
 	int		i;
-	long		v = 0;
+	long		v = 0;		/* variable */
 	long		l1, order = 1;
 	token_type	want;
 	token_type	*source;
@@ -414,7 +428,7 @@ char	*cp;
 		}
 	}
 	if (no_vars(source, n, &v)) {
-		error(_("Current expression contains no variables, there are no extrema."));
+		error(_("Current expression contains no variables."));
 		return false;
 	}
 	if (v == 0) {
@@ -423,7 +437,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(source, n, v)) {
-		error(_("Variable not found."));
+		error(_("Variable not found; the derivative would be zero."));
 		return false;
 	}
 	blt(rhs[i], source, n * sizeof(token_type));
@@ -463,7 +477,7 @@ int
 taylor_cmd(cp)
 char	*cp;
 {
-	long		v = 0;
+	long		v = 0;			/* variable */
 	int		i, j, k, i1;
 	int		level;
 	long		l1, n, order = -1L;
@@ -529,20 +543,17 @@ char	*cp;
 	}
 	blt(rhs[our], source, *nps * sizeof(token_type));
 	our_nrhs = *nps;
+/* Simplify and take the first derivative: */
+	uf_simp(rhs[our], &our_nrhs);
 	if (!differentiate(rhs[our], &our_nrhs, v)) {
 		error(_("Differentiation failed."));
 		return false;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
-		if ((cp = parse_section(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
+		if ((cp = parse_expr(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
 			return false;
 		}
-		if (extra_characters(cp))
-			return false;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
@@ -644,7 +655,9 @@ loop_again:
 				return false;
 			}
 		}
+		symb_flag = true;
 		simpa_side(rhs[our], &our_nrhs, true, true);
+		symb_flag = false;
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
 			i1 = n1;
 			if ((i1 + 1 + our_nrhs) > n_tokens)
@@ -679,12 +692,11 @@ int
 limit_cmd(cp)
 char	*cp;
 {
-	int		i, j, k;
-	long		v = 0;
+	int		i;
+	long		v = 0;			/* variable */
 	token_type	solved_v, want;
 	char		*cp_start;
-	int		infinity_flag, found_den, complex_den, num, den;
-	int		level;
+	int		infinity_flag;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -727,10 +739,7 @@ char	*cp;
 	}
 	if (*cp) {
 		input_column += (cp - cp_start);
-		if (!case_sensitive_flag) {
-			str_tolower(cp);
-		}
-		if ((cp = parse_section(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
+		if ((cp = parse_expr(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
 			return false;
 		}
 	} else {
@@ -740,58 +749,12 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_characters(cp))
-		return false;
 /* copy the current equation to a new equation space and work on the copy: */
 	copy_espace(cur_equation, i);
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
-/* first fully simplify the current equation and group denominators together: */
-	simpa_side(rhs[i], &n_rhs[i], false, false);
-	group_proc(rhs[i], &n_rhs[i]);
-	found_den = complex_den = false;
-	for (j = 1; j < n_rhs[i]; j += 2) {
-		switch (rhs[i][j].token.operatr) {
-		case DIVIDE:
-		case MODULUS:
-			num = den = false;
-			level = rhs[i][j].level;
-			for (k = j + 2;; k += 2) {
-				if (rhs[i][k-1].kind == VARIABLE && rhs[i][k-1].token.variable == v) {
-/* the limit variable was found in a denominator */
-					den = true;
-					found_den = true;
-				}
-				if (k >= n_rhs[i] || rhs[i][k].level <= level) {
-					break;
-				}
-				switch (rhs[i][k].token.operatr) {
-				case DIVIDE:
-				case MODULUS:
-/* a denominator contains a fraction */
-					complex_den = true;
-					break;
-				}
-			}
-			for (k = j - 1; k >= 0 && rhs[i][k].level >= level; k--) {
-				if (rhs[i][k].kind == VARIABLE && rhs[i][k].token.variable == v) {
-/* the limit variable was found in a numerator */
-					num = true;
-				}
-			}
-			if (num && den) {
-				complex_den = true;
-			}
-			break;
-		}
-	}
-	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
-/* in this case, just substitute the limit variable with the limit expression and simplify: */
-		debug_string(0, "Substituting...");
-		subst_var_with_exp(rhs[i], &n_rhs[i], tes, n_tes, v);
-		goto finish_up;
-	}
+
 	debug_string(0, "Solving...");
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
@@ -832,7 +795,6 @@ char	*cp;
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
-finish_up:
 /* symbolically simplify before returning the result: */
 	symb_flag = true;
 	simpa_side(rhs[i], &n_rhs[i], false, false);
