@@ -41,10 +41,12 @@ extern volatile int	screen_rows;
 extern int		finance_option;
 extern int		autosolve;
 extern int		autocalc;
-extern char		special_variable_character;
+extern char		special_variable_characters[200];
 extern int		integer_coefficients;
 extern int		right_associative_power;
+extern int		negate_high_precedence;
 extern int		power_starstar;
+extern int		display_hexadecimal;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
@@ -56,13 +58,14 @@ extern int		partial_flag;
 extern int		symb_flag;
 extern int		high_prec;
 extern int		input_column;
-extern int		sign_flag;
+extern int		sign_cmp_flag;
 extern double		small_epsilon;
 extern double		epsilon;
 extern char		var_str[MAX_VAR_LEN+80];
 extern char		prompt_str[MAX_PROMPT_LEN];
 extern char		*dir_path;
 extern char		*prog_name;
+extern char		*last_history_string;
 
 extern double		unique[];
 extern int		ucnt[];
@@ -75,5 +78,5 @@ extern int		test_mode;
 extern int		quiet_mode;
 extern volatile int	abort_flag;
 
-extern char		*error_str;
+extern const char	*error_str;
 extern char		*result_str;
