@@ -1,7 +1,7 @@
 /*
- * Mathomatic symbolic factorizing routines.
+ * Mathomatic symbolic factorizing routines, not polynomial factoring.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  *
  * There are proper mathematical names for many algebraic rules.
  * I obviously don't know what they are.
@@ -49,14 +49,12 @@ int		*np;
 /*
  * Factor equation side: (a*c + b*c) -> (c*(a + b)).
  *
- * If "v" and "d" equals 0, factor anything,
+ * If "v" and "d" equal 0, factor anything,
  * including identical bases raised to powers (Horner factoring): (x^2 + x) -> (x*(x + 1)).
  * If "d" equals 1.0, only factor identical bases raised to the power of a constant.
  *
- * If "v" is a variable, only factor expressions containing that variable,
- * with no Horner factoring.
- * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised
- * to the power of "d".
+ * If "v" is a variable, only factor expressions containing that variable, with no Horner factoring.
+ * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised to the power of "d".
  *
  * Return true if equation side was modified.
  */
@@ -844,10 +842,11 @@ common_base:
 		equation[i+1].level = level;
 		equation[i+1].kind = CONSTANT;
 		equation[i+1].token.constant = 1.0;
+		binary_parenthesize(equation, i + 2, i);
 	} else {
 		blt(&equation[i+1], &equation[j+len+1], (e2 - j - 1) * sizeof(token_type));
+		binary_parenthesize(equation, i + e2 - j, i);
 	}
-	binary_parenthesize(equation, i + e2 - j, i);
 	blt(&equation[i2+len-1], &equation[e2+len], (*np - (e2 + len)) * sizeof(token_type));
 	*np -= n2 + 1;
 	return true;
