@@ -1,7 +1,7 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -216,18 +216,33 @@ int	n;	/* equation space number */
 	factor_int(rhs[n], &n_rhs[n]);
 }
 
+/*
+ * List an equation side with optional integer factoring.
+ */
+int
+list_factor(equation, np, factor_flag)
+token_type	*equation;
+int		*np;
+int		factor_flag;
+{
+	if (factor_flag || factor_int_flag) {
+		factor_int(equation, np);
+	}
+	return list_proc(equation, *np, false);
+}
+
 /*
  * Neatly factor out coefficients in additive expressions in an equation side.
  * For example: (2*x + 4*y + 6) becomes 2*(x + 2*y + 3).
  *
  * This routine is often necessary because the expression compare (se_compare())
  * does not return a multiplier (except for +/-1.0).
- * This routine is not used during polynomial operations.
  * Normalization done here is required for simplification of algebraic fractions, etc.
  *
  * If "level_code" is 0, all additive expressions are normalized
  * by making at least one coefficient unity (1) by factoring out
- * the smallest constant.  The absolute value of all other coefficients will be >= 1.
+ * the absolute value of the constant coefficient closest to zero.
+ * The absolute value of all other coefficients will be >= 1.
  * If all coefficients are negative, -1 will be factored out, too.
  *
  * If "level_code" is 1, any level 1 additive expression is factored
@@ -369,7 +384,7 @@ break_cont:
 		}
 		i++;
 	}
-	if (integer_coefficients && cogcd != 0.0) {
+	if (integer_coefficients && cogcd != 0.0 && fmod(cogcd, 1.0) == 0.0) {
 		minimum = cogcd;
 	}
 	if (first || count == 0 || (!neg_flag && minimum == 1.0))
