@@ -10,11 +10,12 @@
 
 #include "includes.h"
 
-int		n_tokens = DEFAULT_N_TOKENS;	/* maximum size of expressions, do not change without a restart */
-int		n_equations;			/* number of equation spaces allocated */
-int		cur_equation;			/* current equation space number (origin 0) */
+int		n_tokens = DEFAULT_N_TOKENS;	/* maximum size of expressions, must only be set during startup */
 
-/* expression storage pointers and current length (they go together) */
+int		n_equations,			/* number of equation spaces allocated */
+		cur_equation;			/* current equation space number (origin 0) */
+
+/* expression storage pointers and current length variables (they go together) */
 token_type	*lhs[N_EQUATIONS],		/* The Left Hand Sides of equation spaces */
 		*rhs[N_EQUATIONS];		/* The Right Hand Sides of equation spaces */
 
@@ -30,10 +31,10 @@ int		n_tlhs,				/* number of tokens in tlhs */
 		n_trhs,				/* number of tokens in trhs */
 		n_tes;				/* number of tokens in tes */
 
-token_type	zero_token,			/* the constant 0.0 as a token */
-		one_token;			/* the constant 1.0 as a token */
+token_type	zero_token,			/* the universal constant 0.0 as an expression */
+		one_token;			/* the universal constant 1.0 as an expression */
 
-/* set options */
+/* Set options with their initial values. */
 int		precision = 14;				/* the display precision for doubles (number of digits) */
 int		case_sensitive_flag = true;		/* "set case_sensitive" flag */
 int		factor_int_flag;			/* factor integers when displaying expressions */
@@ -45,10 +46,12 @@ volatile int	screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the termin
 int		finance_option;				/* for displaying dollars and cents */
 int		autosolve = true;			/* Allows solving by typing the variable name at the prompt */
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
-char		special_variable_character = '\\';	/* user defined special character for variable names */
-int		integer_coefficients = true;		/* if true, prefer displaying all integer coefficients */
+char		special_variable_characters[200] = {'\\', '\0'};	/* user defined characters for variable names */
+int		integer_coefficients = false;		/* if true, factor out the GCD of integer coefficients */
 int		right_associative_power;		/* if true, evaluate power operators right to left */
-int		power_starstar;				/* if true, display power operator as "**" */
+int		negate_high_precedence = true;		/* if true, negation (-x) has the highest precedence */
+int		power_starstar;				/* if true, display power operator as "**", otherwise "^" */
+int		display_hexadecimal = false;		/* if true, output constants in hexadecimal */
 #if	!SILENT
 int		debug_level;				/* current debug level */
 #endif
@@ -71,6 +74,9 @@ char		prompt_str[MAX_PROMPT_LEN];	/* temp storage for the prompt string */
 char		*dir_path;			/* directory path to the executable, only set and used in CYGWIN version */
 #endif
 char		*prog_name = "mathomatic";	/* name of this program */
+#if	READLINE
+char		*last_history_string;
+#endif
 
 /* The following are for integer factoring (filled by factor_one()): */
 double		unique[64];		/* storage for the unique prime factors */
@@ -82,17 +88,17 @@ sign_array_type	sign_array;		/* for keeping track of unique "sign" variables */
 FILE		*gfp;			/* global output file pointer */
 jmp_buf		jmp_save;		/* for setjmp(3) to longjmp(3) to when an error happens deep within this code */
 int		test_mode;		/* test mode flag (-t) */
-int		quiet_mode;		/* quiet mode (don't display prompts) */
-int		readline_enabled = true;
-int		partial_flag;		/* true for partial unfactoring of algebraic fractions */
-int		symb_flag;		/* true for "simplify symbolic" */
+int		quiet_mode;		/* quiet mode (-q, don't display prompts) */
+int		readline_enabled = true;	/* set to false (-r) to disable readline */
+int		partial_flag;		/* normally true for partial unfactoring, false for "unfactor fully" */
+int		symb_flag;		/* true during "simplify symbolic" */
 int		high_prec;		/* flag to output constants in higher precision (used when saving equations) */
 int		input_column;		/* current column number on the screen at the beginning of a parse */
-int		sign_flag;		/* true when all "sign" variables are to compare equal */
+int		sign_cmp_flag;		/* true when all "sign" variables are to compare equal */
 int		domain_check;		/* flag to track domain errors in the pow() function */
 int		approximate_roots;	/* true if in calculate command (force approximation of roots like (2^.5)) */
 volatile int	abort_flag;		/* if true, abort current operation */
 
 /* library variables go here */
-char		*error_str;		/* last error string */
+const char	*error_str;		/* last error string */
 char		*result_str;		/* returned result when using as library */
