@@ -27,7 +27,7 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused." },
 #if	!LIBRARY
@@ -35,7 +35,7 @@ static com_type com_list[] = {
 #endif
 {	"compare",	NULL,			compare_cmd,		"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
 {	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the specified equation spaces." },
-{	"derivative",	"differentiate",	derivative_cmd,		"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
 {	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-range]",			"Display equation spaces in multi-line fraction format." },
 #if	!LIBRARY
 {	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD." },
@@ -49,11 +49,11 @@ static com_type com_list[] = {
 {	"factor",	NULL,			factor_cmd,		"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor given integers or factor variables in expressions." },
 {	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert expressions with algebraic fractions into a single fraction." },
 {	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help." },
-{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of an expression (see real command)." },
-{	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials, order times." },
+{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression." },
+{	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials and simplify, order times." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression." },
-{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single line format." },
+{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental)." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single-line format." },
 #if	!LIBRARY
 {	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
 {	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up an equation into smaller, more efficient equations." },
@@ -63,15 +63,15 @@ static com_type com_list[] = {
 #endif
 {	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end." },
 #if	READLINE
-{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push specified equation spaces into the readline history." },
+{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push the specified expressions into the readline history." },
 #endif
 #if	!LIBRARY
-{	"quit",		"exit",			quit_cmd,		"",							"Terminate this program without saving." },
+{	"quit",		"exit",			quit_cmd,		"[exit-value]",						"Terminate this program without saving." },
 #endif
 #if	!SECURE
 {	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of an expression (see imaginary command)." },
+{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression." },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
 {	"roots",	NULL,			roots_cmd,		"root real-part imaginary-part",			"Display all the roots of a complex number." },
@@ -80,25 +80,24 @@ static com_type com_list[] = {
 {	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file." },
 #endif
 {	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Set or display various session options." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
-{	"solve",	NULL,			solve_cmd,		"variable or \"0\"",					"Solve the current equation for a variable or for zero." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] variable or \"0\"",			"Solve the current equation for a variable or for zero." },
 {	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end." },
 #if	!LIBRARY
 {	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show running (grand) total." },
 #endif
 {	"taylor",	NULL,			taylor_cmd,		"variable order point",					"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) expressions." },
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [\"quick\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
 #if	!LIBRARY
 {	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations." },
 #endif
-{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and maximum memory usage." }
+{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and memory usage." }
 };
 
 /*
- * Process equation and expression input in Mathomatic, with no solving
- * and no automatic calculation.
+ * Process mathematical expression input in Mathomatic, with no solving and no automatic calculation.
  *
- * Parse the equation or expression text in "cp" and place in equation space "n".
+ * Simply parse the equation or expression text in "cp" and place the result in equation space "n".
  *
  * Return true if successful.
  */
@@ -124,9 +123,10 @@ char	*cp;
 }
 
 /*
- * Process equation and expression input in Mathomatic.
+ * Process equation and expression input in Mathomatic,
+ * either swapping, selecting, storing, solving (autosolve), or calculating (autocalc).
  *
- * Parse the expression text in "cp" and solve the current equation for it
+ * Parse the mathematical expression text in "cp" and solve the current equation for it
  * or place it in equation space "n" if it is not a solve variable.
  *
  * Return true if successful.
@@ -141,7 +141,7 @@ char	*cp;
 	if (parse_equation(n, cp)) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0) {
 			if (strcmp(cp, "=") == 0 && n_lhs[cur_equation] > 0 && n_rhs[cur_equation] > 0) {
-				debug_string(0, _("Swapping equation sides of the current equation..."));
+				debug_string(0, _("Swapping sides of the current equation..."));
 				n = cur_equation;
 				i = n_lhs[n];
 				blt(tes, lhs[n], n_lhs[n] * sizeof(token_type));
@@ -158,7 +158,7 @@ char	*cp;
 				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
 				    || (lhs[n][0].kind == VARIABLE && (lhs[n][0].token.variable & VAR_MASK) > SIGN)))
 				    || n_rhs[n] == 1) {
-					if (solve(n, cur_equation)) {
+					if (solve_espace(n, cur_equation)) {
 						return return_result(cur_equation);
 					}
 					return false;
@@ -271,7 +271,7 @@ char	*cp;
 		cur_equation = i;
 		return return_result(cur_equation);
 	}
-#if	!SECURE
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /* handle shell escape */
 	if (*cp == '!') {
 		cp1 = getenv("SHELL");
@@ -334,6 +334,9 @@ char	*cp;
 					error(_("Can't open redirected output file for writing."));
 					return false;
 				}
+				if (gfp != stdout) {
+					fclose(gfp);	/* make sure previous redirection file is closed */
+				}
 				gfp = fp;
 			}
 #endif
@@ -341,24 +344,27 @@ char	*cp;
 /* execute the command by calling the command function */
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
-			if (fp) {	/* if output redirected, close output file */
-				if (gfp != stdout)
-					fclose(gfp);
-				else
-					fclose(fp);
+			if (fp && gfp != stdout) {
+				fclose(gfp);
 				gfp = stdout;
 			}
+#endif
+#if	false
+/* This shouldn't display usage while reading in files; only display when the user has typed the command. */
+			if (!rv && debug_level >= 0) {
+				printf("Command usage: %s %s\n", com_list[i].name, com_list[i].usage);
+			}
 #endif
 			return rv;
 		}
 	}
-/* cp is not a command, so parse the expression or equation */
+/* cp is not a command, so parse the expression */
 	i = next_espace();
 	input_column += (cp - cp_start);
 	return process_parse(i, cp);
 }
 
-#if	!SECURE
+#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
 /*
  * Execute a system command.  Note that system(3) requires "/bin/sh".
  *
@@ -366,7 +372,7 @@ char	*cp;
  */
 int
 shell_out(cp)
-char	*cp;
+char	*cp;	/* system command string */
 {
 	int	rv;
 
@@ -462,6 +468,16 @@ next_argument:
 			}
 			return true;
 		}
+		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
+			P("Report bugs to the author via email:");
+			P("\tGeorge Gesslein II");
+			P("\tgeorgegesslein@gmail.com");
+			P("or on the Ubuntu Launchpad website:");
+			P("\thttp://bugs.launchpad.net/mathomatic\n");
+
+			P("Please include the version number.");
+			return true;
+		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
 			P("Command Usage Syntax");
 			P("--------------------");
@@ -477,26 +493,26 @@ next_argument:
 			return true;
 		}
 		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
-			P("Commonly Used Geometric Formulas");
-			P("--------------------------------");
+			P("Commonly used standard (Euclidean) geometric formulas");
+			P("-----------------------------------------------------");
 			P("Triangle of base \"b\" and height \"h\":");
-			P("    area = b*h/2");
+			P("    area = b*h/2\n");
+
 			P("Rectangle of length \"l\" and width \"w\":");
-			P("    area = l*w                    perimeter = 2*l + 2*w");
+			P("    area = l*w                    perimeter = 2*l + 2*w\n");
+
 			P("Trapezoid of parallel sides \"a\" and \"b\", and \"d\" distance between them:");
-			P("    area = d*(a + b)/2");
+			P("    area = d*(a + b)/2\n");
+
 			P("Circle of radius \"r\":");
 			P("    area = pi*r^2                 perimeter = 2*pi*r");
-			P("    area = perimeter*r/2");
+			P("    area = perimeter*r/2\n");
+
 			P("Rectangular solid of length \"l\", width \"w\", and height \"h\":");
-			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h");
+			P("    volume = l*w*h                surface_area = 2*l*w + 2*l*h + 2*w*h\n");
+
 			P("Sphere of radius \"r\":");
-			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2");
-			P("Right circular cylinder of radius \"r\" and height \"h\":");
-			P("    volume = pi*r^2*h             surface_area = 2*pi*r*(h + r)");
-			P("Right circular cone of radius \"r\" and height \"h\":");
-			P("    volume = pi*r^2*h/3");
-			P("    base_surface_area = pi*r^2    side_surface_area = pi*r*(r^2 + h^2)^.5\n");
+			P("    volume = 4/3*pi*r^3           surface_area = 4*pi*r^2\n");
 
 			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
 			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
@@ -504,7 +520,8 @@ next_argument:
 		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
-			P("Operators have precedence decreasing as indicated:");
+			P("Operators have precedence decreasing as indicated:\n");
+
 			P("    - negate");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
@@ -513,16 +530,16 @@ next_argument:
 			P("    = equate\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Predefined constants and variables follow:");
-			P("    e or e# - the universal constant e (2.7182818284...).");
-			P("    pi or pi# - the universal constant pi (3.1415926535...).");
-			P("    i or i# - the imaginary number (square root of -1).");
+			P("Predefined constants and variables follow:\n");
+
+			P("    e or e# - the universal constant e (2.7182818284...)");
+			P("    pi or pi# - the universal constant pi (3.1415926535...)");
+			P("    i or i# - the imaginary number (square root of -1)");
 			P("The above constants may also be used most anywhere variables are required.");
-			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1.");
-			P("    inf - infinity constant (not a variable).\n");
+			P("    sign, sign1, sign2, sign3, ... - variables that can only be +1 or -1");
+			P("    inf - floating point infinity constant (not a variable)\n");
 
 			P("Absolute value notation \"|x|\" and dual polarity \"+/-\" are understood.");
-			P("Full documentation can be found at http://mathomatic.org/");
 			return true;
 		}
 		if (is_all(cp)) {
@@ -537,22 +554,22 @@ next_argument:
 					printf("\n");
 				}
 			}
-			P("Full documentation can be found at http://mathomatic.org/");
+			P("Full documentation can be found at \"http://mathomatic.org/math/doc/\".");
 			return true;
 		}
 		error(_("Unrecognized help topic or command."));
 		return false;
 	}
 	/* default help text: */
-	P("This help command is provided as a quick reference.");
-	P("Full documentation can be found at http://mathomatic.org/");
+	P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
+	P("Type \"help bugs\" for info on how to report bugs found in this program.");
+	P("Type \"help equations\" for help with entering expressions and equations.");
 	P("Type \"help all\" for a summary and syntax of all commands.");
 	P("Type \"help usage\" to display the syntax of all commands.");
-	P("Type \"help equations\" for help with entering expressions and equations.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
 	P("\"help\" or \"?\" followed by command names will give info on those commands.\n");
 
-	P("Available commands:");
+	fprintf(gfp, "These are the %d commands:\n", ARR_CNT(com_list));
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if ((i % 5) == 0)
 			fprintf(gfp, "\n");
