@@ -4,10 +4,20 @@
  * Copyright (C) 1987-2008 George Gesslein II.
  */
 
-/* include files from /usr/include: */
+#define	true	1
+#define	false	0
+
+#if	LIBRARY
+#define	SILENT	true
+#undef	READLINE
+#endif
+
+/* Include files from /usr/include: */
 #include <stdio.h>
 #include <stdlib.h>
+#if	UNIX || CYGWIN
 #include <unistd.h>
+#endif
 #if	UNIX
 #include <libgen.h>
 #endif
@@ -26,7 +36,7 @@
 #include <readline/history.h>
 #endif
 
-/* include files from the current directory: */
+/* Include files from the current directory: */
 #include "am.h"		/* the main include file for Mathomatic, contains tunable parameters */
 #include "complex.h"	/* floating point complex number arithmetic function prototypes */
 #include "proto.h"	/* global function prototypes */
