@@ -3,7 +3,7 @@
  *
  * These are very low level, so they don't do much, polynomials only.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,10 +12,13 @@ static int	integrate_sub();
 static int	laplace_sub();
 static int	inv_laplace_sub();
 
+static int	constant_var_number = 1;	/* makes unique numbers for the constant of integration */
+
 /*
  * Make variable "v" always raised to a power,
  * unless it is on the right side of a power operator.
  */
+void
 make_powers(equation, np, v)
 token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
@@ -230,15 +233,19 @@ char	*cp;
 	long		v = 0;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
-	int		def_flag;
+	int		definite_flag = false, constant_flag = false;
 	double		d1, integrate_order = 1.0;
+	char		temp_buf[50];
 
 	if (current_not_defined()) {
 		return false;
 	}
 	i = next_espace();
-	def_flag = (strcmp_tospace(cp, "definite") == 0);
-	if (def_flag) {
+	if (strcmp_tospace(cp, "definite") == 0) {
+		definite_flag = true;
+		cp = skip_param(cp);
+	} else if (strcmp_tospace(cp, "constant") == 0) {
+		constant_flag = true;
 		cp = skip_param(cp);
 	}
 	if (n_rhs[cur_equation]) {
@@ -253,9 +260,11 @@ char	*cp;
 		np = &n_lhs[i];
 	}
 	if (*cp) {
-		cp = parse_var2(&v, cp);
-		if (cp == NULL) {
-			return false;
+		if (isvarchar(*cp)) {
+			cp = parse_var2(&v, cp);
+			if (cp == NULL) {
+				return false;
+			}
 		}
 		if (*cp) {
 			integrate_order = strtod(cp, &cp);
@@ -264,7 +273,8 @@ char	*cp;
 			error(_("The order must be a positive integer."));
 			return false;
 		}
-	} else {
+	}
+	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
@@ -280,9 +290,32 @@ char	*cp;
 			error(_("Integration failed, not a polynomial."));
 			return false;
 		}
+		if (constant_flag) {
+			if (n1 + 2 > n_tokens) {
+				error_huge();
+			}
+			for (j = 0; j < n1; j++) {
+				dest[j].level++;
+			}
+			dest[n1].kind = OPERATOR;
+			dest[n1].level = 1;
+			dest[n1].token.operatr = PLUS;
+			n1++;
+			dest[n1].kind = VARIABLE;
+			dest[n1].level = 1;
+			snprintf(temp_buf, sizeof(temp_buf), "C_%d", constant_var_number);
+			if (parse_var(&dest[n1].token.variable, temp_buf) == NULL) {
+				return false;
+			}
+			n1++;
+			constant_var_number++;
+			if (constant_var_number < 0) {
+				constant_var_number = 1;
+			}
+		}
 		simp_loop(dest, &n1);
 	}
-	if (def_flag) {
+	if (definite_flag) {
 		my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 		if (!get_expr(tlhs, &n_tlhs)) {
 			return false;
@@ -601,7 +634,6 @@ char	*cp;
 		if (!prompt_var(&v))
 			return false;
 	}
-#if	!SILENT
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
@@ -613,9 +645,8 @@ char	*cp;
 		}
 	}
 	if (singularity) {
-		printf(_("Warning: Singularity detected, result of numerical integration may be wrong.\n"));
+		warning(_("Singularity detected, result of numerical integration may be wrong."));
 	}
-#endif
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
 		return false;
@@ -639,9 +670,9 @@ char	*cp;
 	}
 #if	!SILENT
 	if (trap_flag) {
-		printf(_("Approximating definite integral using the trapezoid method (%d partitions)...\n"), iterations);
+		printf(_("Approximating the definite integral using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
-		printf(_("Approximating definite integral using Simpson's rule (%d partitions)...\n"), iterations);
+		printf(_("Approximating the definite integral using Simpson's rule (%d partitions)...\n"), iterations);
 	}
 #endif
 	subst_constants(source, nps);
@@ -755,7 +786,7 @@ char	*cp;
 		approximate_roots = false;
 
 		if (exp_contains_infinity(dest, n1)) {
-			error(_("Integration failed because result contains infinity or NaN."));
+			error(_("Integration failed because result contains infinity or NaN (a singularity)."));
 			return false;
 		}
 		/* detect an ever growing result: */
