@@ -8,14 +8,14 @@
 #include "includes.h"
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
-#define MODULUS_STRING	" % "	/* string displayed for the modulus binary operator */
+#define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
 
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
 
 static int	flist();
 static int	flist_recurse();
 
-/* ANSI color code array */
+/* ANSI color code array for 8 color ANSI, we don't use black */
 static int	carray[] = {
 	32,	/* green (default color) */
 	33,	/* yellow */
@@ -26,8 +26,8 @@ static int	carray[] = {
 	37	/* white */
 };
 
-/* HTML color array */
-static char	*html_carray[] = {
+/* Bright HTML color array */
+static char	*bright_html_carray[] = {
 	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
 	"#FF0000",
@@ -37,12 +37,25 @@ static char	*html_carray[] = {
 	"#0000FF"
 };
 
+/* Dim HTML color array */
+static char	*html_carray[] = {
+	"green",	/* green (default color) */
+	"olive",
+	"red",
+	"maroon",
+	"purple",
+	"teal",
+	"navy"
+};
+
 /* global variables for flist() */
 int	cur_line;	/* current line */
 int	cur_pos;	/* current position in the current line on the screen */
 
 /*
+ * Reset terminal attributes function.
  * Turn color off if color mode is on.
+ * Subsequent output will have no color.
  */
 void
 reset_attr()
@@ -60,6 +73,7 @@ reset_attr()
 
 /*
  * Set the current color on the display.
+ * Subsequent output will have the color in carray[color].
  * Range is 0 to INT_MAX.
  */
 void
@@ -75,20 +89,21 @@ int	color;
 			if (cur_color >= 0) {
 				printf("</font>");
 			}
-			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
-		} else {
 			if (bold_colors) {
-				printf("\033[1;%dm", carray[color%ARR_CNT(carray)]);
+				printf("<font color=\"%s\">", bright_html_carray[color%ARR_CNT(bright_html_carray)]);
 			} else {
-				printf("\033[0;%dm", carray[color%ARR_CNT(carray)]);
+				printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
 			}
+		} else {
+			printf("\033[%d;%dm", bold_colors, carray[color%ARR_CNT(carray)]);
 		}
 		cur_color = color;
 	}
 }
 
 /*
- * Set the normal text color.
+ * Set normal text color for subsequent output.
+ * Currently, the normal color (0) is green.
  */
 void
 default_color()
@@ -173,13 +188,13 @@ token_type	*p2;
 int		n2;
 {
 	if (debug_level >= level) {
-		printf(_("level %d: "), level);
+		fprintf(gfp, _("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
-			printf(EQUATE_STRING);
+			fprintf(gfp, EQUATE_STRING);
 			list_proc(p2, n2, false);
 		}
-		printf("\n");
+		fprintf(gfp, "\n");
 	}
 }
 #endif
@@ -299,7 +314,7 @@ int	lang_code;	/* language code */
 }
 
 /*
- * Display an expression in single line format.
+ * Display an expression in single-line format.
  * Use color if available.
  * The expression is not modified.
  *
@@ -345,9 +360,17 @@ int		export_flag;	/* flag for exportable format (usually false) */
 					len += fprintf(gfp, "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += fprintf(gfp, "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					len += fprintf(gfp, "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
-				len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
+				if (display_hexadecimal) {
+					len += fprintf(gfp, "%a", p1[i].token.constant);
+				} else {
+					len += fprintf(gfp, "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			break;
 		case VARIABLE:
@@ -515,7 +538,11 @@ int		export_flag;
 					snprintf(buf, sizeof(buf), "%.20g", p1[i].token.constant);
 				}
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
 				snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
 			}
@@ -685,6 +712,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 			if (equation[i].token.constant == 0.0) {
 				equation[i].token.constant = 0.0; /* fix -0 */
 			}
+/* note: here we will need to parenthesize negative numbers to make -2**x work the same with Python: */
 			if (int_flag) {
 				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
@@ -700,7 +728,11 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 					}
 				}
 			}
-			len += fprintf(gfp, "%s", buf);
+			if (equation[i].token.constant < 0) {
+				len += fprintf(gfp, "(%s)", buf);
+			} else {
+				len += fprintf(gfp, "%s", buf);
+			}
 			break;
 		case VARIABLE:
 			len += list_var(equation[i].token.variable, language + 1);
@@ -968,7 +1000,7 @@ check_again:
 			if (oflag) {
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					if (html_flag) {
+					if (html_flag && gfp == stdout) {
 						fprintf(gfp, "&mdash;");
 					} else {
 						fprintf(gfp, "-");
@@ -994,9 +1026,17 @@ check_again:
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				if (p1[i].token.constant < 0.0) {
+					len += snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+				}
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				if (display_hexadecimal) {
+					len += snprintf(buf, sizeof(buf), "%a", p1[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.*g", precision, p1[i].token.constant);
+				}
 			}
 			if (oflag)
 				fprintf(gfp, "%s", buf);
@@ -1014,7 +1054,7 @@ check_again:
 				len += 3;
 				break;
 			case MINUS:
-				if (html_flag) {
+				if (html_flag && gfp == stdout) {
 					cp = " &minus; ";
 				} else {
 					cp = " - ";
@@ -1022,7 +1062,7 @@ check_again:
 				len += 3;
 				break;
 			case TIMES:
-				if (html_flag) {
+				if (html_flag && gfp == stdout) {
 					cp = "&middot;";
 				} else {
 					cp = "*";
