@@ -1,6 +1,6 @@
 /*
- * This file contains main() and the startup code for Mathomatic.
- * Leave this file out when compiling for the symbolic math library.
+ * This file contains main() and the startup code for Mathomatic,
+ * which is an algebraic manipulator written entirely in C.
  *
  * Copyright (C) 1987-2008 George Gesslein II.
  *
@@ -35,8 +35,13 @@
 
 #if	!LIBRARY
 #include "includes.h"
+#if	UNIX || CYGWIN
 #include <sys/ioctl.h>
 #include <termios.h>
+#endif
+#if	TRAP_FPERRORS	/* Do not use this define!  Currently does not work. */
+#include <fenv.h>
+#endif
 
 static void usage();
 static int set_signals();
@@ -66,6 +71,11 @@ char	**argv;
 	/* initialize the global variables */
 	init_gvars();
 	gfp = stdout;
+#if	READLINE
+	using_history();		/* initialize readline history */
+	stifle_history(500);		/* maximum of 500 entries */
+	rl_inhibit_completion = true;	/* turn off readline file name completion */
+#endif
 
 	/* process command line options */
 	while ((i = getopt(argc, argv, "qrtchuvm:")) >= 0) {
@@ -95,7 +105,8 @@ char	**argv;
 			test_mode = true;
 			break;
 		case 'u':
-			setbuf(stdout, NULL);	/* make standard output unbuffered */
+			setbuf(stdout, NULL);	/* make output unbuffered */
+			setbuf(stderr, NULL);
 			break;
 		case 'v':
 			version_report();
@@ -107,13 +118,20 @@ char	**argv;
 	if (test_mode || html_flag) {
 		screen_columns = 0;
 		screen_rows = 0;
+#if	UNIX || CYGWIN
 	} else {
 		get_screen_size();
+#endif
 	}
 	if (!init_mem()) {
 		fprintf(stderr, _("%s: Not enough memory.\n"), prog_name);
 		exit(2);
 	}
+#if	READLINE && !SECURE
+	if (readline_enabled) {
+		read_history(NULL);	/* restore readline history of previous session */
+	}
+#endif
 	if (html_flag) {
 		printf("<pre>\n");
 	}
@@ -123,10 +141,8 @@ char	**argv;
 #endif
 		printf(_("Mathomatic version %s (www.mathomatic.org)\n"), VERSION);
 		printf(_("Copyright (C) 1987-2008 George Gesslein II.\n"));
-		printf(_("This is free software; see the source for copying conditions.  There is NO\n"));
-		printf(_("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"));
-		printf(_("%d equation spaces available, %lld kilobytes per equation space.\n"),
-		    N_EQUATIONS, (long long) n_tokens * sizeof(token_type) * 2LL / 1000LL);
+		printf(_("%d equation spaces available, %ldK per equation space.\n"),
+		    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 	}
 #if	!SECURE
 	/* read the user options initialization file */
@@ -140,7 +156,7 @@ char	**argv;
 		color_flag = !color_flag;
 	}
 	if (!quiet_mode && color_flag) {
-		printf(_("%s color mode enabled.\n"), html_flag ? "HTML" : "ANSI");
+		printf(_("%s%s color mode enabled.\n"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
 	}
 	if ((i = setjmp(jmp_save)) != 0) {
 		/* for error handling */
@@ -188,17 +204,17 @@ char	**argv;
 static void
 usage()
 {
-	fprintf(stderr, "Mathomatic version %s - automatic algebraic manipulator\n\n", VERSION);
-	fprintf(stderr, "Usage: %s [ options ] [ input_files ]\n\n", prog_name);
-	fprintf(stderr, "Options:\n");
-	fprintf(stderr, "  -c            Toggle color mode.\n");
-	fprintf(stderr, "  -h            Enable HTML output mode.\n");
-	fprintf(stderr, "  -m number     Specify a memory size multiplier.\n");
-	fprintf(stderr, "  -q            Set quiet mode (don't display prompts).\n");
-	fprintf(stderr, "  -r            Disable readline.\n");
-	fprintf(stderr, "  -t            Set test mode.\n");
-	fprintf(stderr, "  -u            Set unbuffered output.\n");
-	fprintf(stderr, "  -v            Display version number, compile flags, and memory usage.\n");
+	printf("Mathomatic version %s - automatic algebraic manipulator\n\n", VERSION);
+	printf("Usage: %s [ options ] [ input_files ]\n\n", prog_name);
+	printf("Options:\n");
+	printf("  -c            Toggle color mode.\n");
+	printf("  -h            Enable HTML output mode.\n");
+	printf("  -m number     Specify a memory size multiplier.\n");
+	printf("  -q            Set quiet mode (don't display prompts).\n");
+	printf("  -r            Disable readline.\n");
+	printf("  -t            Set test mode.\n");
+	printf("  -u            Set unbuffered output.\n");
+	printf("  -v            Display version number, compile flags, and memory usage.\n");
 	exit(2);
 }
 
@@ -214,10 +230,16 @@ set_signals()
 
 	if (signal(SIGFPE, fphandler) == SIG_ERR)
 		rv = false;
+#if	TRAP_FPERRORS
+	if (feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW) < 0)
+		rv = false;
+#endif
 	if (signal(SIGINT, inthandler) == SIG_ERR)
 		rv = false;
+#if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
 		rv = false;
+#endif
 #if	TIMEOUT_SECONDS
 	if (signal(SIGALRM, alarmhandler) == SIG_ERR)
 		rv = false;
@@ -290,17 +312,26 @@ load_rc()
 
 /*
  * Floating point exception handler.
- * Usually doesn't work in most operating systems.
+ * I do not know the proper function calls that should be here to make the GNU TRAP_FPERRORS define work.
+ * Currently it goes in an endless loop when an FP exception occurs and TRAP_FPERRORS is true,
+ * because when this function returns, the exception immediately happens again.
+ * Therefore, do not define TRAP_FPERRORS!
  */
 void
 fphandler(sig)
 int	sig;
 {
-/*	error(_("Floating point exception.")); */
+#if	TRAP_FPERRORS
+	warning(_("Floating point exception."));
+/*
+	feenableexcept(FE_INVALID | FE_DIVBYZERO | FE_OVERFLOW);
+	signal(SIGFPE, fphandler);
+*/
+#endif
 }
 
 /*
- * Control-C (interrupt) handler.
+ * Control-C (interrupt) signal handler.
  * Interrupts processing and returns to main prompt through a polling mechanism.
  * If it can't, repeated calls terminate this program.
  */
@@ -335,8 +366,9 @@ int	sig;
 	exit_program(1);
 }
 
+#if	UNIX || CYGWIN
 /*
- * Get the screen (window) width and height.
+ * Get the screen (window) width and height from the operating system.
  */
 void
 get_screen_size()
@@ -356,7 +388,7 @@ get_screen_size()
 }
 
 /*
- * Window resize handler.
+ * Window resize signal handler.
  */
 void
 resizehandler(sig)
@@ -364,6 +396,7 @@ int	sig;
 {
 	get_screen_size();
 }
+#endif
 
 /*
  * Exit this program and return to the Operating System.
@@ -376,6 +409,11 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 	if (html_flag) {
 		printf("</pre>\n");
 	}
+#if	READLINE && !SECURE
+	if (readline_enabled) {
+		write_history(NULL);	/* save readline history in ~/.history */
+	}
+#endif
 	exit(exit_value);
 }
 #endif
