@@ -34,7 +34,7 @@ char	*cp;	/* error message */
 #if	!SILENT
 	int	 i;
 
-	if (isatty(0) && point_flag) {
+	if (isatty(0) && point_flag && !html_flag) {
 		for (i = 0; i < (input_column + cnt); i++) {
 			printf(" ");
 		}
@@ -51,7 +51,7 @@ int
 isvarchar(ch)
 int	ch;
 {
-	return(ch == '_' || (special_variable_character && ch == special_variable_character) || isalpha(ch));
+	return(ch == '_' || (ch && strchr(special_variable_characters, ch)) || isalpha(ch));
 }
 
 /*
@@ -104,9 +104,10 @@ int		*np;
 		if (equation[i].token.operatr == NEGATE) {
 			/* change negate operator to -1 times the operand: */
 			equation[i].token.operatr = TIMES;
-			/* make negate top priority by parenthesizing it first: */
-			binary_parenthesize(equation, *np, i);
-			/* comment out the above line to make negate the same priority as times/divide */
+			if (negate_high_precedence) {
+				/* make negate top priority by parenthesizing it first: */
+				binary_parenthesize(equation, *np, i);
+			}	/* otherwise negate has same priority as times/divide */
 		}
 	}
 }
@@ -114,6 +115,8 @@ int		*np;
 /*
  * Parenthesize most operators so that expression evaluation is in the correct order.
  * Gives different operators on the same level in an expression the correct priority.
+ * Similar operators on the same level are always evaluated or grouped left to right,
+ * except for the power operator.
  *
  * organize() should be called after this to remove unneeded parentheses.
  */
@@ -132,13 +135,13 @@ int		*np;		/* pointer to expression length */
 	}
 
 	if (right_associative_power) {
-		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (right to left) */
+		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (for right to left evaluation) */
 			if (equation[i].token.operatr == POWER) {
 				binary_parenthesize(equation, *np, i);
 			}
 		}
 	} else {
-		for (i = 1; i < *np; i += 2) {		/* count up (left to right) */
+		for (i = 1; i < *np; i += 2) {		/* count up (for left to right evaluation) */
 			if (equation[i].token.operatr == POWER) {
 				binary_parenthesize(equation, *np, i);
 			}
@@ -214,7 +217,6 @@ char		*cp;		/* string to parse */
 		case '\t':
 			continue;
 		case '(':
-		case '[':
 		case '{':
 			cur_level++;
 			if (operand) {
@@ -222,11 +224,10 @@ char		*cp;		/* string to parse */
 			}
 			continue;
 		case ')':
-		case ']':
 		case '}':
 			cur_level--;
 			if (cur_level <= 0 || (abs_count > 0 && cur_level < abs_array[abs_count-1])) {
-				put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: too many )."));
+				put_up_arrow((int) (cp - cp_start), _("Unmatched parenthesis: too many )"));
 				return(NULL);
 			}
 			if (!operand) {
@@ -283,7 +284,7 @@ char		*cp;		/* string to parse */
 				goto syntax_error;
 			}
 			if (cp[1] == '!' && cp[2] != '!') {
-				warning(_("Multifactorial not implemented, use \"(x!)!\" for the factorial of x factorial."));
+				warning(_("Multifactorial not implemented, using \"(x!)!\" for the factorial of x factorial."));
 			}
 			equation[n].level = cur_level;
 			equation[n].kind = OPERATOR;
@@ -385,7 +386,7 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (*cp == '-' && !(isdigit(cp[1]) || cp[1] == '.')) {
+			if (*cp == '-' && (!negate_high_precedence || !(isdigit(cp[1]) || cp[1] == '.'))) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = -1.0;
 				equation[n].level = cur_level;
@@ -450,7 +451,7 @@ p_out:
 		goto syntax_error;
 	}
 	if (cur_level != 1) {
-		put_up_arrow((int) (cp - cp_start), _("Unmatched parentheses: missing )."));
+		put_up_arrow((int) (cp - cp_start), _("Unmatched parenthesis: missing )"));
 		return(NULL);
 	}
 	while (*cp == '=')
@@ -512,7 +513,7 @@ char	*cp;
 	}
 	buf[i] = '\0';
 	if (strcasecmp(buf, INFINITY_NAME) == 0) {
-		error(_("Invalid variable."));
+		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
 		vtmp = SIGN;
@@ -552,6 +553,10 @@ char	*cp;
 			*vp = V_PI;
 			return(cp1);
 		}
+		if (is_all(buf)) {
+			error(_("\"all\" is a reserved word and may not be used as a variable name."));
+			return(NULL);
+		}
 		vtmp = 0;
 		for (i = 0; var_names[i]; i++) {
 			if ((*strcmpfunc)(buf, var_names[i]) == 0) {
