@@ -3,14 +3,18 @@
  * Includes polynomial and smart division, polynomial factoring, etc.
  * Globals tlhs[] and trhs[] are wiped out by most of these routines.
  *
- * The polynomial division and GCD routines here are not recursive, due to static
- * storage areas.  This limits the polynomial GCD to univariate operation.
+ * The polynomial division and GCD routines here are not recursive, due to static storage areas.
+ * This limits the polynomial GCD routine to mostly univariate operation.
+ * This also does not allow their use during solving.
+ * So far, these limitations have been a good thing, making Mathomatic faster and more stable.
  *
  * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
+#define	DISABLE_DIVISION	false
+
 #define	REMAINDER_IS_ZERO()	(n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
 
 /* These are a non-standard size and must only be used for temp storage: */
@@ -21,13 +25,8 @@ int		n_quotient;			/* length of expression in quotient[] */
 token_type	gcd_divisor[DIVISOR_SIZE];	/* static data area for polynomial GCD routine */
 int		len_d;				/* length of expression in gcd_divisor[] */
 
-static int	do_gcd();
-static int	pf_recurse();
-static int	pf_sub();
-static int	mod_recurse();
-static int	polydiv_recurse();
-static int	pdiv_recurse();
-static int	poly_div_sub();
+static int	do_gcd(), pf_recurse(), pf_sub(), mod_recurse(), polydiv_recurse(), pdiv_recurse(), poly_div_sub();
+static int	find_highest_count();
 static void	save_factors();
 
 /*
@@ -42,7 +41,9 @@ sort_type	*p1, *p2;
 
 /*
  * Return true if passed expression is strictly a single polynomial term in variable v.
- * The exponent may be anything, as long as it doesn't contain the variable.
+ * The general form of a polynomial term is c*(v^d)
+ * The coefficient (c) and exponent (d) may be any expression,
+ * as long as it doesn't contain the variable v.
  */
 static int
 poly_in_v_sub(p1, n, v, allow_divides)
@@ -95,8 +96,9 @@ int		allow_divides;	/* if true, allow division by variable */
 }
 
 /*
- * Return true if passed expression is a polynomial in variable v.
- * The exponents may be anything, as long as they don't contain the variable.
+ * Return true if passed expression is a polynomial in the given variable v.
+ * The coefficients and exponents may be anything, as long as they don't contain the variable v.
+ * The passed expression should be fully unfactored, for a proper determination.
  */
 int
 poly_in_v(p1, n, v, allow_divides)
@@ -181,8 +183,11 @@ int		*np, loc, level;
  */
 static int
 pf_sub(equation, np, loc, len, level)
-token_type	*equation;
-int		*np, loc, len, level;
+token_type	*equation;	/* equation side holding the possible polynomial to factor */
+int		*np,		/* pointer to length of equation side */
+		loc,		/* index of start of polynomial in equation side */
+		len;		/* length of polynomial */
+int		level;		/* level of additive operators in polynomial */
 {
 	token_type	*p1;
 	int		modified = false, symbolic_modified = false;
@@ -198,14 +203,17 @@ int		*np, loc, len, level;
 	int		vc, cnt;
 	sort_type	va[MAX_VARS];
 	double		d;
+	int		old_partial;
 
 	debug_string(3, "Entering pf_sub().");
+	old_partial = partial_flag;
 	loc2 = loc1 = loc;
 	find_greatest_power(&equation[loc1], len, &v, &d, &j, &k, &div_flag);
 	if (v == 0)
 		return false;
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+		partial_flag = old_partial;
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		if (i == 13) {	/* critical error code */
 			longjmp(jmp_save, i);
@@ -217,7 +225,9 @@ int		*np, loc, len, level;
 	for (count = 1;; count++) {
 		blt(trhs, &equation[loc1], len * sizeof(token_type));
 		n_trhs = len;
+		partial_flag = false;
 		uf_simp(trhs, &n_trhs);
+		partial_flag = old_partial;
 		if (level1_plus_count(trhs, n_trhs) < 2) {
 /* must be at least 2 level 1 additive operators to be factorable */
 			goto skip_factor;
@@ -244,6 +254,7 @@ int		*np, loc, len, level;
 			va[vc].count = cnt;
 		}
 		side_debug(3, &equation[loc1], len);
+		side_debug(3, trhs, n_trhs);
 /* find a valid polynomial base variable "v": */
 		cnt = -1;
 		if (v) {
@@ -264,17 +275,40 @@ int		*np, loc, len, level;
 				cnt = va[i].count;
 			}
 		}
-		if (cnt <= 1 || v == 0)
+		if (cnt <= 1)
 			goto skip_factor;
-		blt(tlhs, trhs, n_trhs * sizeof(token_type));
-		n_tlhs = n_trhs;
-		if (!differentiate(tlhs, &n_tlhs, v)) {
-			break;
+		if (v == 0) {
+#if	true
+			goto skip_factor;
+#else
+/* no polynomial variables found, try differentiating with respect to each variable until one works */
+			for (i = 0; i < vc; i++) {
+				if ((va[i].v & VAR_MASK) <= SIGN) {
+					continue;
+				}
+				v = va[i].v;
+				blt(tlhs, trhs, n_trhs * sizeof(token_type));
+				n_tlhs = n_trhs;
+				if (differentiate(tlhs, &n_tlhs, v)) {
+					break;
+				}
+				v = 0;
+			}
+			if (v == 0) {
+				goto skip_factor;
+			}
+#endif
+		} else {
+			blt(tlhs, trhs, n_trhs * sizeof(token_type));
+			n_tlhs = n_trhs;
+			if (!differentiate(tlhs, &n_tlhs, v)) {
+				goto skip_factor;
+			}
 		}
 #if	!SILENT
 		if (debug_level >= 3) {
 			list_var(v, 0);
-			printf(_("Differentiation successful using variable (%s).\n"), var_str);
+			fprintf(gfp, _("Differentiation successful using variable (%s).\n"), var_str);
 		}
 #endif
 		simp_loop(tlhs, &n_tlhs);
@@ -349,9 +383,9 @@ next_v:
 			n_tlhs = k;
 #if	!SILENT
 			if (debug_level >= 3) {
-				printf(_("Trying factor: "));
+				fprintf(gfp, _("Trying factor: "));
 				list_proc(tlhs, n_tlhs, false);
-				printf("\n");
+				fprintf(gfp, "\n");
 			}
 #endif
 			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) == 0)
@@ -567,7 +601,7 @@ long		v;
 		blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 		len_d = n_tlhs;
 		if (poly_div(larger, llen, gcd_divisor, len_d, &v) != 2) {
-			warning("Polynomial GCD found, but larger divide failed.");
+			warning("Polynomial GCD found, but larger divide failed in poly_gcd().");
 			return 0;
 		}
 	}
@@ -633,7 +667,7 @@ long		v;
 		if (!level1_plus_count(gcd_divisor, len_d))
 			return 0;
 		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
-			warning("Polynomial GCD found, but smaller divide failed.");
+			warning("Polynomial GCD found, but smaller divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -645,7 +679,7 @@ long		v;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		if (poly_div(larger, llen, tlhs, n_tlhs, &v) != 2) {
-			warning("Polynomial GCD found, but larger divide failed.");
+			warning("Polynomial GCD found, but larger divide failed in poly2_gcd().");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -956,7 +990,7 @@ int		*np;
 int		poly_flag;
 int		quick_flag;
 {
-	int	rv;
+	int	rv = false;
 
 	if (quick_flag)
 		group_proc(equation, np);
@@ -976,7 +1010,7 @@ int		*np, loc, level, code;
 	int	op, op2, last_op2;
 	int	len1, len2, real_len1;
 	long	v;
-	int	rv;
+	int	rv = 0;
 	int	flag, power_flag, zero_remainder;
 
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
@@ -1071,7 +1105,7 @@ next_thingy:
 					if (power_flag) {
 						k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
 					} else {
-						k = (var_count(tlhs, n_tlhs) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
+						k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
 					}
 					if (k) {
 						for (k = 0; k < n_tlhs; k++)
@@ -1155,25 +1189,32 @@ token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 long		*vp;		/* variable pointer to base variable */
 {
-	int		i, j;
-	int		old_partial;
+	int		i;
+	int		rv;
+	int		old_partial, old_symb;
 	jmp_buf		save_save;
 
+	if (DISABLE_DIVISION)
+		return false;
 	old_partial = partial_flag;
-	partial_flag = false;	/* We want full unfactoring. */
+	old_symb = symb_flag;
+	partial_flag = false;	/* We want full unfactoring during polynomial division. */
+//	symb_flag = true;
 	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+	if ((i = setjmp(jmp_save)) != 0) {	/* Trap errors so we almost always return normally. */
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		partial_flag = old_partial;
+		symb_flag = old_symb;
 		if (i == 13) {	/* critical error code */
 			longjmp(jmp_save, i);
 		}
 		return false;
 	}
-	j = poly_div_sub(d1, len1, d2, len2, vp);
+	rv = poly_div_sub(d1, len1, d2, len2, vp);
 	blt(jmp_save, save_save, sizeof(jmp_save));
 	partial_flag = old_partial;
-	return j;
+	symb_flag = old_symb;
+	return rv;
 }
 
 /*
@@ -1219,7 +1260,7 @@ long		*vp;
 	/* Display debugging info. */
 	if (debug_level >= 3) {
 		list_var(*vp, 0);
-		printf(_("poly_div() starts using variable (%s):\n"), var_str);
+		fprintf(gfp, _("poly_div() starts using base variable (%s):\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
@@ -1388,6 +1429,8 @@ int		len2;		/* length of divisor */
 	int		dcount = 0;		/* divisor term count */
 	int		flag;
 
+	if (DISABLE_DIVISION)
+		return false;
 	blt(trhs, d1, len1 * sizeof(token_type));
 	n_trhs = len1;
 	blt(tlhs, d2, len2 * sizeof(token_type));
@@ -1662,13 +1705,11 @@ int		*tp1, *lentp1;
 }
 
 /*
- * This routine automatically finds the best normal variable to do
- * polynomial division with.
+ * This routine automatically finds the best normal variable to do polynomial division with.
  *
- * Returns 0 if nothing was found, otherwise variable is returned
- * in "*vp1".
+ * Returns 0 if nothing was found, otherwise the selected base variable is returned in *vp1.
  */
-int
+static int
 find_highest_count(p1, n1, p2, n2, vp1)
 token_type	*p1;		/* pointer to dividend expression */
 int		n1;		/* length of dividend */
