@@ -77,7 +77,7 @@ main(int argc, char *argv[])
 }
 
 void
-allocate_triangle()
+allocate_triangle(void)
 {
 	int	i;
 
@@ -91,7 +91,7 @@ allocate_triangle()
 }
 
 void
-calculate_triangle()
+calculate_triangle(void)
 {
 	int	i, j;
 
@@ -107,7 +107,7 @@ calculate_triangle()
 }
 
 void
-display_triangle()
+display_triangle(void)
 {
 	int		i, j;
 	int		len;
@@ -172,9 +172,9 @@ center_buf(int line_number, int cell_size)
 }
 
 void
-usage()
+usage(void)
 {
-	fprintf(stderr, "Usage: %s [number-of-lines]\n\n", prog_name);
-	fprintf(stderr, "Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
+	printf("Usage: %s [number-of-lines]\n\n", prog_name);
+	printf("Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
 	exit(2);
 }
