@@ -262,7 +262,7 @@ elim_factor(long double arg)
  * Generate and display consecutive prime numbers.
  */
 void
-generate_primes()
+generate_primes(void)
 {
 	int		n, j;
 	long double	count, ii, vv;
@@ -377,23 +377,25 @@ test_pal(long double d, long double base)
 }
 
 void
-usage()
+usage(void)
 {
-	fprintf(stderr, "\nUsage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
-	fprintf(stderr, "Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
-	fprintf(stderr, "If \"twin\" is specified, output only twin primes.\n");
-	fprintf(stderr, "If \"pal\" is specified, output only palindromic primes.\n");
-	fprintf(stderr, "The palindromic base may be specified, the default is base 10.\n");
+	printf("Prime number generator version 1.0\n");
+	printf("Usage: %s [start [stop]] [\"twin\"] [\"pal\" [base]]\n\n", prog_name);
+	printf("Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
+	printf("If \"twin\" is specified, output only twin primes.\n");
+	printf("If \"pal\" is specified, output only palindromic primes.\n");
+	printf("The palindromic base may be specified, the default is base 10.\n");
 	exit(2);
 }
 
 void
-usage2()
+usage2(void)
 {
-	fprintf(stderr, "\nUsage: %s [options] [start [stop]]\n\n", prog_name);
-	fprintf(stderr, "Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
-	fprintf(stderr, "Options:\n");
-	fprintf(stderr, "  -t               Output only twin primes.\n");
-	fprintf(stderr, "  -p base          Output only palindromic primes.\n");
+	printf("Prime number generator version 1.0\n");
+	printf("Usage: %s [options] [start [stop]]\n\n", prog_name);
+	printf("Generate batches of consecutive prime numbers up to %Lg.\n", max_integer);
+	printf("Options:\n");
+	printf("  -t               Output only twin primes.\n");
+	printf("  -p base          Output only palindromic primes.\n");
 	exit(2);
 }
