@@ -1,18 +1,18 @@
 /*
  * Global function prototypes for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 /* command function list */
 int		clear_cmd(), quit_cmd(), list_cmd(), simplify_cmd(), help_cmd(), eliminate_cmd();
 int		fraction_cmd(), unfactor_cmd(), compare_cmd(), extrema_cmd();
 int		read_cmd(), display_cmd(), calculate_cmd(), solve_cmd();
-int		factor_cmd(), derivative_cmd(), replace_cmd();
+int		factor_cmd(), derivative_cmd(), replace_cmd(), approximate_cmd();
 int		save_cmd(), taylor_cmd(), limit_cmd(), echo_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd(), version_cmd();
 int		edit_cmd(), real_cmd(), imaginary_cmd(), tally_cmd();
-int		roots_cmd(), set_cmd(), code_cmd(), optimize_cmd(), push_cmd();
+int		roots_cmd(), set_cmd(), variables_cmd(), code_cmd(), optimize_cmd(), push_cmd();
 int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
 
 /* various functions that don't return int */
