@@ -11,13 +11,139 @@
  * Most of the stack usage is related this value,
  * this many pointers are pushed onto the stack with each level of parentheses while comparing two expressions.
  */
-#define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 10)
+#define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 6)
 
 static int	org_recurse();
 static int	const_recurse();
 static int	compare_recurse();
 static int	order_recurse();
 
+/*
+ * Fix up levels of parentheses in an equation side.
+ * Remove unnecessary parentheses and put similar operators on the same level.
+ *
+ * This is the inner-most loop in Mathomatic, make it fast.
+ */
+void
+organize(equation, np)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+{
+	if (*np <= 0 || (*np & 1) != 1) {
+		error("Internal error: organize() called with bad expression size.");
+		longjmp(jmp_save, 13);
+	}
+	if (*np > n_tokens) {
+		error("Internal error: expression array overflow detected.");
+		longjmp(jmp_save, 13);
+	}
+	org_recurse(equation, np, 0, 1, NULL);
+}
+
+static inline void
+org_up_level(bp, ep, level, invert)
+token_type	*bp, *ep;
+int		level, invert;
+{
+	if (invert) {
+		for (; bp <= ep; bp++) {
+			bp->level--;
+			if (bp->level == level && bp->kind == OPERATOR) {
+				switch (bp->token.operatr) {
+				case PLUS:
+					bp->token.operatr = MINUS;
+					break;
+				case MINUS:
+					bp->token.operatr = PLUS;
+					break;
+				case TIMES:
+					bp->token.operatr = DIVIDE;
+					break;
+				case DIVIDE:
+					bp->token.operatr = TIMES;
+					break;
+				}
+			}
+		}
+	} else {
+		for (; bp <= ep; bp++) {
+			bp->level--;
+		}
+	}
+}
+
+/*
+ * Recurse through every sub-expression in "equation", starting at "loc",
+ * moving up levels to "level" of parentheses.
+ */
+static int
+org_recurse(equation, np, loc, level, elocp)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+int		loc, level, *elocp;
+{
+	token_type	*p1, *bp, *ep;
+	int		op, sub_op;
+	int		i;
+	int		eloc;
+	int		sub_eloc;
+	int		min1;
+	int		invert;
+
+	bp = &equation[loc];
+	ep = &equation[*np];
+	min1 = bp->level;
+	for (p1 = bp + 1; p1 < ep; p1 += 2) {
+		if (p1->level < min1) {
+			if (p1->level < level)
+				break;
+			min1 = p1->level;
+		}
+	}
+	ep = p1;
+	eloc = (ep - equation) - 1;
+	if (elocp)
+		*elocp = eloc;
+	if (eloc == loc) {
+		bp->level = max(level - 1, 1);
+		return 0;
+	}
+	if (min1 > level) {
+		for (p1 = bp; p1 < ep; p1++)
+			p1->level -= min1 - level;
+	}
+	op = 0;
+	for (p1 = bp + 1; p1 < ep; p1 += 2) {
+		if (p1->level == level) {
+			op = p1->token.operatr;
+			break;
+		}
+	}
+	for (i = loc; i <= eloc; i += 2) {
+		if (equation[i].level > level) {
+			sub_op = org_recurse(equation, np, i, level + 1, &sub_eloc);
+			switch (sub_op) {
+			case PLUS:
+			case MINUS:
+				if (op != PLUS && op != MINUS)
+					break;
+				invert = (i - 1 >= loc && equation[i-1].token.operatr == MINUS);
+				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
+				break;
+			case TIMES:
+			case DIVIDE:
+				if (op != TIMES && op != DIVIDE)
+					break;
+				invert = (i - 1 >= loc && equation[i-1].token.operatr == DIVIDE);
+				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
+				break;
+			}
+			i = sub_eloc;
+		}
+	}
+	return op;
+}
+
 /*
  * The most basic simplification loop.
  * Just does constant simplification.
@@ -72,7 +198,7 @@ simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
-	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
+	simp_ssub(equation, np, 0L, /* 0.0 */ 1.0, true, true, 1);
 }
 
 /*
@@ -106,7 +232,8 @@ long		v;		/* variable to factor, 0 for all variables */
 }
 
 /*
- * Convert expression with any algebraic fractions into a single fraction.
+ * Convert expressions with any algebraic fractions into a single simple fraction.
+ * Used by the fraction command.
  */
 void
 frac_side(equation, np)
@@ -116,12 +243,14 @@ int		*np;		/* pointer to length of equation side */
 	if (*np == 0)
 		return;
 	do {
-		simpb_side(equation, np, false, 2);
+		simp_loop(equation, np);
+		uf_allpower(equation, np);
+		simp_ssub(equation, np, 0L, 1.0, false, true, 2);
 	} while (super_factor(equation, np, 2));
 }
 
 /*
- * Simplify and approximate for the calculate command.
+ * Simplify and approximate for the calculate command and for more successful comparisons.
  * Includes complex number simplification.
  */
 void
@@ -202,8 +331,8 @@ void
 simpa_side(equation, np, quick_flag, frac_flag)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of the equation side */
-int		quick_flag;	/* "simplify quick" option */
-int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxima's ratsimp() */
+int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
 {
 	int		i;
 	int		flag, poly_flag = true;
@@ -211,20 +340,23 @@ int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxim
 
 	if (*np <= 1)	/* no need to simplify a single constant or variable */
 		return;
+	side_debug(2, equation, *np);
+#if	true	/* polynomial factoring adds to processing time, but sometimes simplifies more */
 	simp_loop(equation, np);
 	poly_factor(equation, np);
+#endif
 	simpb_side(equation, np, true, 1);
 	simp_ssub(equation, np, 0L, 0.0, true, true, 1);
 	rationalize(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
 	simp_loop(equation, np);
-	/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
+/* Here is the only place in Mathomatic that we do modulus (%) simplification: */
 	uf_repeat(equation, np);
 	do {
 		elim_loop(equation, np);
 	} while (mod_simp(equation, np));
-	/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
+/* Here we try to simplify out unnecessary negative constants and imaginary numbers: */
 	simp_i(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
@@ -253,29 +385,36 @@ int		frac_flag;	/* if true, simplify to the ratio of two polynomials, like Maxim
 			simp_ssub(equation, np, 0L, 1.0, false, true, 1);
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
-	if (quick_flag) {
-do_quick:
+
+/* Here we do the greatest expansion; if it fails, do less expansion. */
+	partial_flag = frac_flag;
+	n_tlhs = *np;
+	blt(tlhs, equation, n_tlhs * sizeof(token_type));
+	blt(save_save, jmp_save, sizeof(jmp_save));
+	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (i == 13) {	/* critical error code */
+			longjmp(jmp_save, i);
+		}
+		/* an error occurred, restore the original expression */
+		*np = n_tlhs;
+		blt(equation, tlhs, n_tlhs * sizeof(token_type));
+		debug_string(1, "Simplify not expanding fully.");
+		partial_flag = true;	/* expand less */
 		uf_tsimp(equation, np);
 	} else {
-		n_tlhs = *np;
-		blt(tlhs, equation, n_tlhs * sizeof(token_type));
-		blt(save_save, jmp_save, sizeof(jmp_save));
-		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
-			blt(jmp_save, save_save, sizeof(jmp_save));
-			if (i == 13) {	/* critical error code */
-				longjmp(jmp_save, i);
-			}
-			/* an error occurred, restore the original expression */
-			*np = n_tlhs;
-			blt(equation, tlhs, n_tlhs * sizeof(token_type));
-			goto do_quick;	/* skip power expansion */
+		if (quick_flag) {
+			uf_tsimp(equation, np);
+		} else {
+			/* expand powers of 2 and higher, might result in error_huge() trap */
+			do {
+				uf_repeat(equation, np);
+			} while (uf_tsimp(equation, np));
 		}
-		/* expand powers of 2 and higher, might result in error_huge() trap */
-		do {
-			uf_repeat(equation, np);
-		} while (uf_tsimp(equation, np));
 		blt(jmp_save, save_save, sizeof(jmp_save));
 	}
+	partial_flag = true;
+
 	simpb_side(equation, np, true, 1);
 	debug_string(1, "Simplify result before applying polynomial operations:");
 	side_debug(1, equation, *np);
@@ -303,13 +442,11 @@ do_quick:
 	}
 	simpb_side(equation, np, true, 1);
 	simp_constant_power(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	simp_ssub(equation, np, /* 0L */ MATCH_ANY, 1.0, true, true, 1);
 	unsimp_power(equation, np);
 	make_fractions(equation, np);
 	factor_power(equation, np);
-	partial_flag = frac_flag;
 	uf_tsimp(equation, np);
-	partial_flag = true;
 	make_fractions(equation, np);
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
@@ -365,7 +502,11 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 	sort_type	va[MAX_VARS];
 
 	simp_loop(equation, np);
+#if	false
+	uf_power(equation, np);
+#else
 	uf_allpower(equation, np);
+#endif
 	last_v = 0;
 	for (vc = 0; vc < ARR_CNT(va); vc++) {
 		cnt = 0;
@@ -523,131 +664,6 @@ int		fc_level;
 	} while (factor_divide(equation, np, v, 0.0));
 }
 
-/*
- * Fix up levels of parentheses in an equation side.
- * Remove unnecessary parentheses and put similar operators on the same level.
- *
- * This is the inner-most loop in Mathomatic, make it fast.
- */
-organize(equation, np)
-token_type	*equation;	/* equation side pointer */
-int		*np;		/* pointer to length of equation side */
-{
-	if (*np <= 0 || (*np & 1) != 1) {
-		error("Internal error: Organize called with bad expression size.");
-		longjmp(jmp_save, 13);
-	}
-	if (*np > n_tokens) {
-		error("Internal error: Unchecked expression array overflow detected.");
-		longjmp(jmp_save, 13);
-	}
-	org_recurse(equation, np, 0, 1, NULL);
-}
-
-static inline void
-org_up_level(bp, ep, level, invert)
-token_type	*bp, *ep;
-int		level, invert;
-{
-	if (invert) {
-		for (; bp <= ep; bp++) {
-			bp->level--;
-			if (bp->level == level && bp->kind == OPERATOR) {
-				switch (bp->token.operatr) {
-				case PLUS:
-					bp->token.operatr = MINUS;
-					break;
-				case MINUS:
-					bp->token.operatr = PLUS;
-					break;
-				case TIMES:
-					bp->token.operatr = DIVIDE;
-					break;
-				case DIVIDE:
-					bp->token.operatr = TIMES;
-					break;
-				}
-			}
-		}
-	} else {
-		for (; bp <= ep; bp++) {
-			bp->level--;
-		}
-	}
-}
-
-/*
- * Recurse through every sub-expression in "equation", starting at "loc",
- * moving up levels to "level" of parentheses.
- */
-static int
-org_recurse(equation, np, loc, level, elocp)
-token_type	*equation;	/* equation side pointer */
-int		*np;		/* pointer to length of equation side */
-int		loc, level, *elocp;
-{
-	token_type	*p1, *bp, *ep;
-	int		op, sub_op;
-	int		i;
-	int		eloc;
-	int		sub_eloc;
-	int		min1;
-	int		invert;
-
-	bp = &equation[loc];
-	ep = &equation[*np];
-	min1 = bp->level;
-	for (p1 = bp + 1; p1 < ep; p1 += 2) {
-		if (p1->level < min1) {
-			if (p1->level < level)
-				break;
-			min1 = p1->level;
-		}
-	}
-	ep = p1;
-	eloc = (ep - equation) - 1;
-	if (elocp)
-		*elocp = eloc;
-	if (eloc == loc) {
-		bp->level = max(level - 1, 1);
-		return 0;
-	}
-	if (min1 > level) {
-		for (p1 = bp; p1 < ep; p1++)
-			p1->level -= min1 - level;
-	}
-	op = 0;
-	for (p1 = bp + 1; p1 < ep; p1 += 2) {
-		if (p1->level == level) {
-			op = p1->token.operatr;
-			break;
-		}
-	}
-	for (i = loc; i <= eloc; i += 2) {
-		if (equation[i].level > level) {
-			sub_op = org_recurse(equation, np, i, level + 1, &sub_eloc);
-			switch (sub_op) {
-			case PLUS:
-			case MINUS:
-				if (op != PLUS && op != MINUS)
-					break;
-				invert = (i - 1 >= loc && equation[i-1].token.operatr == MINUS);
-				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
-				break;
-			case TIMES:
-			case DIVIDE:
-				if (op != TIMES && op != DIVIDE)
-					break;
-				invert = (i - 1 >= loc && equation[i-1].token.operatr == DIVIDE);
-				org_up_level(&equation[i], &equation[sub_eloc], level, invert);
-				break;
-			}
-			i = sub_eloc;
-		}
-	}
-	return op;
-}
-
 /*
  * Convert (x^n)^m to x^(n*m).
  *
@@ -666,42 +682,44 @@ int		*np;
 	double		numerator, denominator;
 
 	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			level = equation[i].level;
-			left_level = 1;
-			for (j = i - 2; j >= 0; j -= 2) {
-				if (equation[j].level < level) {
-					left_level = equation[j].level;
-					break;
-				}
+		if (equation[i].token.operatr != POWER)
+			continue;
+		level = equation[i].level;
+		left_level = 1;
+		for (j = i - 2; j >= 0; j -= 2) {
+			if (equation[j].level < level) {
+				left_level = equation[j].level;
+				break;
 			}
-			for (j = i + 2; j < *np; j += 2) {
-				if (equation[j].level <= level) {
-					if (left_level <= equation[j].level && equation[j].token.operatr == POWER) {
-						if (!symb_flag && (i + 2) == j && equation[j-1].kind == CONSTANT) {
-							f_to_fraction(equation[j-1].token.constant, &numerator, &denominator);
+		}
+		for (j = i + 2; j < *np; j += 2) {
+			if (equation[j].level <= level) {
+				if (left_level <= equation[j].level && equation[j].token.operatr == POWER) {
+					if (!symb_flag) {
+						if ((i + 2) == j && equation[i+1].kind == CONSTANT
+						    && equation[j].level == equation[j+1].level
+						    && equation[j+1].kind == CONSTANT) {
+							f_to_fraction(equation[i+1].token.constant, &numerator, &denominator);
 							if (fmod(numerator, 2.0) == 0.0) {
-								if (equation[j].level == equation[j+1].level
-								    && equation[j+1].kind == CONSTANT) {
-									f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
-									if (fmod(denominator, 2.0) == 0.0) {
-										break;
-									}
+								f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
+								if (fmod(denominator, 2.0) == 0.0) {
+									break;
 								}
 							}
-						}
-						equation[j].token.operatr = TIMES;
-						for (k = i + 1; k < j; k++)
-							equation[k].level++;
-						diff = (level - equation[j].level) + 1;
-						level = equation[j].level;
-						for (k = j; k < *np && equation[k].level >= level; k++) {
-							equation[k].level += diff;
-						}
-						modified = true;
+						} else
+							break;
 					}
-					break;
+					equation[j].token.operatr = TIMES;
+					for (k = i + 1; k < j; k++)
+						equation[k].level++;
+					diff = (level - equation[j].level) + 1;
+					level = equation[j].level;
+					for (k = j; k < *np && equation[k].level >= level; k++) {
+						equation[k].level += diff;
+					}
+					modified = true;
 				}
+				break;
 			}
 		}
 	}
@@ -827,7 +845,9 @@ int		*np, i;
 }
 
 /*
- * Simplify constant^(constant*x).
+ * Simplify constant^(constant*x) to (constant^constant)^x.
+ * I am not sure if this is a good thing, because it changes the base of the exponent.
+ * The tests don't require this routine.
  *
  * Return true if equation side was modified.
  */
@@ -839,9 +859,11 @@ int		*np;
 	int	modified = false;
 	int	i;
 
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			modified |= sp2_sub(equation, np, i);
+	if (symb_flag) {
+		for (i = 1; i < *np; i += 2) {
+			if (equation[i].token.operatr == POWER) {
+				modified |= sp2_sub(equation, np, i);
+			}
 		}
 	}
 	return modified;
@@ -941,20 +963,25 @@ int		iflag;		/* produce imaginary numbers flag */
  * Do the floating point arithmetic for Mathomatic.
  *
  * Return true if successful.
+ * domain_check must be set to false after this.
  */
-static int
+int
 calc(op1p, k1p, op2, k2)
-int	*op1p;	/* pointer to operator 1, sometimes modified */
-double	*k1p;	/* pointer to operand 1 and where to store the result */
-int	op2;	/* operator 2 */
-double	k2;	/* operand 2 */
+int	*op1p;	/* Pointer to operator 1, which comes immediately before operand 1 if this operator exists. */
+double	*k1p;	/* Pointer to operand 1 and where to store the result of the calculation. */
+int	op2;	/* Operator 2; always exists and comes immediately before operand 2, and usually after operand 1. */
+double	k2;	/* Operand 2; ignored for unary operators. */
 {
 	int	op1;
 	double	d, d1, d2;
 
 	domain_check = false;
 	errno = 0;
-	op1 = *op1p;
+	if (op1p) {
+		op1 = *op1p;
+	} else {
+		op1 = 0;
+	}
 	switch (op2) {
 	case PLUS:
 	case MINUS:
@@ -990,14 +1017,17 @@ double	k2;	/* operand 2 */
 			*k1p *= k2;
 		} else {
 			if (op1 == DIVIDE) {
+				check_divide_by_zero(*k1p);
 				*k1p = k2 / *k1p;
 				*op1p = TIMES;
 			} else if (op2 == DIVIDE) {
+				check_divide_by_zero(k2);
 				*k1p = *k1p / k2;
 			}
 		}
 		break;
 	case IDIVIDE:
+		check_divide_by_zero(k2);
 		modf(*k1p / k2, k1p);
 		break;
 	case MODULUS:
@@ -1016,16 +1046,22 @@ double	k2;	/* operand 2 */
 		break;
 	case POWER:
 		if (*k1p < 0.0 && fmod(k2, 1.0) != 0.0) {
+			/* it's probably imaginary; pow() will give a domain error, so skip these calculations */
 			break;
 		}
+#if	false
 		if (*k1p == 0.0 && k2 < 0.0) {
-			error(_("Divide by zero error (0 raised to negative power)."));
+			error(_("Divide by zero (0 raised to negative power)."));
 			longjmp(jmp_save, 2);
 		}
+#endif
 		domain_check = true;
 		if (*k1p == 0.0 && k2 == 0.0) {
-			warning(_("0^0 encountered, might be considered undefined."));
-			d = 1.0;	/* some people don't know 0^0 */
+			warning(_("0^0 encountered, might be considered indeterminate."));
+			d = 1.0;	/* 0^0 = 1 */
+		} else if (*k1p == 0 && k2 < 0.0) {
+			warning(_("Divide by zero (0 raised to negative power)."));
+			d = INFINITY;
 		} else {
 			d = pow(*k1p, k2);
 			if (preserve_roots && !approximate_roots) {
@@ -1071,9 +1107,9 @@ int		*np, loc, level, iflag;
 {
 	int		loc1, old_loc;
 	int		const_count = 0;
-	int		op, zero = 0;
+	int		op;
 	int		modified = false;
-	double		d, d1, d2, numerator, denominator;
+	double		d1, d2, d3, numerator, denominator;
 	complexs	cv, p;
 
 	loc1 = old_loc = loc;
@@ -1097,14 +1133,14 @@ beginning:
 				continue;
 			}
 			op = equation[loc-1].token.operatr;
-			d = equation[loc1].token.constant;
+			d1 = equation[loc1].token.constant;
 			d2 = equation[loc].token.constant;
-			if (calc((loc1 <= old_loc) ? &zero : &equation[loc1-1].token.operatr, &d, op, d2)) {
+			if (calc((loc1 <= old_loc) ? NULL : &equation[loc1-1].token.operatr, &d1, op, d2)) {
 				if (op == POWER && !domain_check) {
 					if (!f_to_fraction(d2, &numerator, &denominator)) {	/* if irrational power */
 						if (!iflag || (preserve_roots && !approximate_roots))
 							return modified;
-						cv.re = d;
+						cv.re = d1;
 						cv.im = 0.0;
 						p.re = d2;
 						p.im = 0.0;
@@ -1136,13 +1172,13 @@ beginning:
 						return true;
 					}
 					errno = 0;
-					d1 = pow(-d, d2);
+					d3 = pow(-d1, d2);
 					check_err();
 					if (!always_positive(denominator)) {
 						if (!always_positive(numerator)) {
-							d1 = -d1;
+							d3 = -d3;
 						}
-						d = d1;
+						d1 = d3;
 						goto not_imaginary;
 					}
 					if (!iflag)
@@ -1155,7 +1191,7 @@ beginning:
 					if (d2 == 0.5) {
 						equation[loc1].level = level + 1;
 						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = -d;
+						equation[loc1].token.constant = -d1;
 						loc1++;
 						equation[loc1].level = level + 1;
 						equation[loc1].kind = OPERATOR;
@@ -1174,7 +1210,7 @@ beginning:
 					} else {
 						equation[loc1].level = level;
 						equation[loc1].kind = CONSTANT;
-						equation[loc1].token.constant = d1;
+						equation[loc1].token.constant = d3;
 						loc1++;
 						equation[loc1].level = level;
 						equation[loc1].kind = OPERATOR;
@@ -1195,7 +1231,7 @@ beginning:
 					return true;
 				} else {
 not_imaginary:
-					equation[loc1].token.constant = d;
+					equation[loc1].token.constant = d1;
 					modified = true;
 					domain_check = false;
 					blt(&equation[loc-1], &equation[loc+1], (*np - (loc + 1)) * sizeof(token_type));
@@ -1384,6 +1420,7 @@ int		*np;		/* pointer to length of equation side */
 			if (p2->level == level && p2->kind == CONSTANT) {
 /* Replace division by a constant with times its reciprocal. */
 				f_to_fraction(p2->token.constant, &numerator, &denominator);
+				check_divide_by_zero(numerator);
 				p2->token.constant = denominator / numerator;
 				p1->token.operatr = TIMES;
 				continue;
@@ -1505,7 +1542,7 @@ int		n2, l2, *diff_signp;
 		}
 		switch (p1->kind) {
 		case VARIABLE:
-			if (sign_flag && (p1->token.variable & VAR_MASK) == SIGN) {
+			if (sign_cmp_flag && (p1->token.variable & VAR_MASK) == SIGN) {
 				return((p2->token.variable & VAR_MASK) == SIGN);
 			} else {
 				return(p1->token.variable == p2->token.variable);
@@ -1608,6 +1645,7 @@ no_op2:
 			opa2[oc2] = pv1 + 1;
 			used[oc2] = false;
 			if (++oc2 >= ARR_CNT(opa2)) {
+				debug_string(1, "Expression too big to compare, because MAX_COMPARE_TERMS may be too small.");
 				return false;	/* expression too big to compare */
 			}
 		}
@@ -1803,7 +1841,15 @@ int		*np;
 							modified = true;
 						}
 					} else if ((equation[i-1].token.variable & VAR_MASK) == SIGN) {
-						equation[i+1].token.constant = fmod(equation[i+1].token.constant, 2.0);
+						if (fmod(denominator, 2.0) == 1.0) {
+							/* odd root, make root (denominator) 1 */
+							numerator = fmod(numerator, 2.0);
+							/* all sign^2 now translate to 1 */
+							if (numerator != equation[i+1].token.constant) {
+								equation[i+1].token.constant = numerator;
+								modified = true;
+							}
+						}
 					}
 				}
 			}
@@ -2082,7 +2128,7 @@ int		*np, loc, level;
 	}
 	return modified;
 corrupt:
-	error(_("Expression is corrupt!  Please file a bug report."));
+	error(_("Expression is corrupt!  Please send a bug report."));
 	longjmp(jmp_save, 13);
 }
 
