@@ -21,17 +21,26 @@ static int	repeat_count;
  * Return true if successful.
  */
 int
-solve(want, have)
+solve_espace(want, have)
 int	want;	/* equation number containing what to solve for */
 int	have;	/* equation number to solve */
 {
-	int	rv;
+	int	rv;	/* return value */
 
-	if (n_lhs[want] > 0) {
-		rv = solve_sub(lhs[want], n_lhs[want], lhs[have], &n_lhs[have], rhs[have], &n_rhs[have]);
+	if (n_lhs[want]) {
+		if (n_rhs[want]) {
+			/* Something in the LHS and RHS of equation number "want". */
+			error(_("This program will only solve for a single variable or for zero."));
+			rv = false;
+		} else {
+			/* Normal solve: */
+			rv = solve_sub(lhs[want], n_lhs[want], lhs[have], &n_lhs[have], rhs[have], &n_rhs[have]);
+		}
 	} else {
+		/* Solve variable was preceded by an equals sign, solve using reversed equation sides: */
 		rv = solve_sub(rhs[want], n_rhs[want], rhs[have], &n_rhs[have], lhs[have], &n_lhs[have]);
 	}
+	/* Free the "want" equation space: */
 	n_lhs[want] = 0;
 	n_rhs[want] = 0;
 #if	!SILENT
@@ -49,23 +58,23 @@ int	have;	/* equation number to solve */
  * to the LHS (via transposition), then moving everything not containing the variable to the
  * RHS.  Many tricks are used, and this routine works very well.
  *
- * tlhs and trhs are used by this routine.
+ * Globals tlhs[] and trhs[] are used to hold the actual equation while manipulating.
  *
  * Returns a positive integer if successful, with the result placed in the passed LHS and RHS.
  * Returns 1 for normal success.
  * Returns 2 if successful and a solution was zero and removed.
  * Returns 0 on failure.  Might succeed at a numeric solve.
- * Returns -1 if the equation is an identity.
+ * Returns -1 if solving for a variable and the equation is an identity.
  * Returns -2 if unsolvable in all realms.
  */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
 token_type	*wantp;		/* expression to solve for */
-int		wantn;		/* size of expression to solve for (currently should always be 1) */
+int		wantn;		/* length of expression to solve for (currently should always be 1) */
 token_type	*leftp;		/* LHS of equation */
-int		*leftnp;	/* size of LHS */
+int		*leftnp;	/* pointer to length of LHS */
 token_type	*rightp;	/* RHS of equation */
-int		*rightnp;	/* size of RHS */
+int		*rightnp;	/* pointer to length of RHS */
 {
 	int		i, j;
 	int		found, found_count;
@@ -73,7 +82,7 @@ int		*rightnp;	/* size of RHS */
 	int		diff_sign;
 	int		op, op_kind;
 	token_type	*p1, *b1, *ep;
-	long		v;
+	long		v;			/* variable to solve for */
 	int		need_flip;
 	int		uf_flag = false;	/* unfactor flag */
 	int		qtries = 0;
@@ -100,11 +109,8 @@ int		*rightnp;	/* size of RHS */
 	}
 	if (wantp->kind == VARIABLE) {
 		v = wantp->token.variable;
-		if (v == 0) {
-			error(_("Solve variable is null!"));
-			return false;
-		}
 		if (!found_var(trhs, n_trhs, v) && !found_var(tlhs, n_tlhs, v)) {
+			/* variable v is 0 or not found */
 			error(_("Variable not found."));
 			return false;
 		}
@@ -349,6 +355,7 @@ zero_simp:
 						debug_string(1, _("Solve variable moved back to RHS, quitting solve routine."));
 						return false;
 					}
+					/* The following code determines if we have an identity: */
 					calc_simp(tlhs, &n_tlhs);
 					calc_simp(trhs, &n_trhs);
 					if (se_compare(tlhs, n_tlhs, trhs, n_trhs, &diff_sign) && !diff_sign) {
@@ -458,9 +465,9 @@ zero_simp:
 						qtries = 0;	/* might be quadratic after removing solution */
 						success = 2;
 #if	!SILENT
-						printf(_("Removing possible solution: \""));
+						fprintf(gfp, _("Removing possible solution: \""));
 						list_proc(b1, 1, false);
-						printf(" = 0\".\n");
+						fprintf(gfp, " = 0\".\n");
 #endif
 					} else {
 						debug_string(1, _("Juggling..."));
@@ -499,12 +506,11 @@ zero_simp:
 					if (inc_count > MAX_RAISE_POWER)
 						return false;
 #if	!SILENT
-					printf(_("Raising both sides to the power of %g and unfactoring...\n"), 1.0 / b1->token.constant);
+					printf(_("Raising both equation sides to the power of %g and unfactoring...\n"), 1.0 / b1->token.constant);
 #endif
 					zero_solved = false;
 					qtries = 0;
 					if (!increase(b1->token.constant, v)) {
-						error(_("Failed to raise both sides to a power."));
 						return false;
 					}
 					uf_flag = true;
@@ -565,7 +571,7 @@ long	v;
 	ufactor(tlhs, &n_tlhs);
 	partial_flag = true;
 	simp_ssub(tlhs, &n_tlhs, v, d, true, false, 2);
-	simp_ssub(tlhs, &n_tlhs, 0L, 0.0, true, true, 2);
+	simp_ssub(tlhs, &n_tlhs, 0L, /* 0.0 */ 1.0, true, true, 2);
 lonely:
 	ep = &tlhs[n_tlhs];
 	len2 = len1 = 0;
@@ -982,7 +988,11 @@ big_bbreak:
 	n_trhs = len;
 	simp_loop(trhs, &n_trhs);
 	list_tdebug(2);
-	uf_simp(trhs, &n_trhs);
+#if	true
+	uf_tsimp(trhs, &n_trhs);	/* don't unfactor result so much, just unfactor what will be unfactored anyway */
+#else
+	uf_simp(trhs, &n_trhs);		/* totally unfactor, usually faster but result may be unwanted */
+#endif
 	simps_side(trhs, &n_trhs, false);
 	list_tdebug(1);
 #if	!SILENT
@@ -1064,15 +1074,15 @@ int		*side2np;	/* pointer to the length of "side2p" */
 			return false;
 		if (!get_constant(operandp, operandn, &d2))
 			return false;
-		debug_string(1, _("Taking logarithm of both sides:"));
+		debug_string(1, _("Taking logarithm of both equation sides:"));
 		*side2np = 1;
 		side2p->level = 1;
 		side2p->kind = CONSTANT;
 		errno = 0;
 		side2p->token.constant = log(d1) / log(d2);
-		check_err();
 		blt(side1p, p1 + 1, (*side1np - (operandn + 1)) * sizeof(token_type));
 		*side1np -= operandn + 1;
+		check_err();
 		return true;
 	}
 	if (op == MODULUS) {
@@ -1087,40 +1097,40 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	if (debug_level > 0) {
 		switch (op) {
 		case PLUS:
-			printf(_("Subtracting"));
+			fprintf(gfp, _("Subtracting"));
 			break;
 		case MINUS:
-			printf(_("Adding"));
+			fprintf(gfp, _("Adding"));
 			break;
 		case TIMES:
-			printf(_("Dividing both sides of the equation by"));
+			fprintf(gfp, _("Dividing both sides of the equation by"));
 			break;
 		case DIVIDE:
-			printf(_("Multiplying both sides of the equation by"));
+			fprintf(gfp, _("Multiplying both sides of the equation by"));
 			break;
 		case POWER:
-			printf(_("Raising both sides of the equation to the power of"));
+			fprintf(gfp, _("Raising both sides of the equation to the power of"));
 			break;
 		case MODULUS:
-			printf(_("Applying inverse modulus of"));
+			fprintf(gfp, _("Applying inverse modulus of"));
 			break;
 		}
-		printf(" \"");
+		fprintf(gfp, " \"");
 		if (op == POWER)
-			printf("1/(");
+			fprintf(gfp, "1/(");
 		list_proc(operandp, operandn, false);
 		switch (op) {
 		case PLUS:
-			printf(_("\" from both sides of the equation:\n"));
+			fprintf(gfp, _("\" from both sides of the equation:\n"));
 			break;
 		case MINUS:
 		case MODULUS:
-			printf(_("\" to both sides of the equation:\n"));
+			fprintf(gfp, _("\" to both sides of the equation:\n"));
 			break;
 		case POWER:
-			printf(")");
+			fprintf(gfp, ")");
 		default:
-			printf("\":\n");
+			fprintf(gfp, "\":\n");
 			break;
 		}
 	}
