@@ -9,7 +9,9 @@
 static int	unf_sub();
 
 /*
- * Unfactor times and divide only and simplify.
+ * Unfactor times and divide only (products of sums) and simplify.
+ *
+ * Return true if equation side was modified.
  */
 int
 uf_tsimp(equation, np)
@@ -20,7 +22,7 @@ int		*np;
 
 	rv = uf_times(equation, np);
 	simp_loop(equation, np);
-	if (uf_times(equation, np)) {
+	while (uf_times(equation, np)) {
 		rv = true;
 		simp_loop(equation, np);
 	}
@@ -30,6 +32,7 @@ int		*np;
 /*
  * Unfactor power only.
  * (a * b)^c -> a^c * b^c
+ *
  * Return true if equation side is modified.
  */
 int
@@ -50,6 +53,7 @@ int		*np;
 /*
  * Unfactor power only.
  * a^(b + c) -> a^b * a^c
+ *
  * Return true if equation side is modified.
  */
 int
@@ -125,8 +129,6 @@ uf_simp(equation, np)
 token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
 {
-	if (*np == 0)
-		return;
 	uf_tsimp(equation, np);
 	uf_power(equation, np);
 	uf_repeat(equation, np);
@@ -169,28 +171,10 @@ int		*np;
 {
 	int	i, j;
 	int	level;
-	int	plus_flag;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == DIVIDE) {
 			level = equation[i].level;
-#if	false
-			if (partial_flag != 2) {
-				plus_flag = false;
-				for (j = i + 2; j < *np; j += 2) {
-					if (equation[j].level <= level)
-						break;
-					switch (equation[j].token.operatr) {
-					case PLUS:
-					case MINUS:
-						plus_flag = true;
-						break;
-					}
-				}
-				if (!plus_flag)
-					continue;
-			}
-#endif
 			for (j = i - 1; j >= 0; j--) {
 				if (equation[j].level < level)
 					break;
@@ -201,9 +185,9 @@ int		*np;
 }
 
 /*
- * Unfactor times and divide only.
+ * Unfactor times and divide only (products of sums like (a+b)*(c+d)).
  * (a + b)*c -> a*c + b*c
- * If "partial_flag", don't unfactor (a+b)/(c+d).
+ * If partial_flag is true, don't expand (a+b)/(c+d) nor (a+b)/c.
  *
  * Return true if equation side was modified.
  */
